! ================================================================================================================================
! MODULE       : stomate_litter
!
! CONTACT      : orchidee-help _at_ ipsl.jussieu.fr
!
! LICENCE      : IPSL (2006)
! This software is governed by the CeCILL licence see ORCHIDEE/ORCHIDEE_CeCILL.LIC
!
!>\BRIEF       Update litter and lignine content after litter fall and 
!! calculating litter decomposition.      
!!
!!\n DESCRIPTION: None
!!
!! RECENT CHANGE(S): None
!!
!! REFERENCE(S)	: None
!!
!! SVN          :
!! $HeadURL$
!! $Date$
!! $Revision$
!! \n
!_ ================================================================================================================================

MODULE stomate_litter

  ! modules used:

  USE ioipsl_para
  USE stomate_data
  USE constantes
  USE constantes_soil
  USE pft_parameters
  USE function_library, ONLY : check_mass_balance, biomass_to_lai, cc_to_biomass

  IMPLICIT NONE

  ! private & public routines

  PRIVATE
  PUBLIC littercalc,littercalc_clear, deadleaf

  LOGICAL, SAVE             :: firstcall_litter = .TRUE.       !! first call
!$OMP THREADPRIVATE(firstcall_litter)

CONTAINS

!! ================================================================================================================================
!! SUBROUTINE   : littercalc_calc
!!
!!\BRIEF        Set the flag ::firstcall_litter to .TRUE. and as such activate section
!! 1.1 of the subroutine littercalc (see below).
!!
!! DESCRIPTION  : None
!!
!! RECENT CHANGE(S) : None
!!
!! MAIN OUTPUT VARIABLE(S) : None
!!
!! REFERENCE(S) : None
!!
!! FLOWCHART : None
!! \n
!_ ================================================================================================================================

  SUBROUTINE littercalc_clear
    firstcall_litter =.TRUE.
  END SUBROUTINE littercalc_clear


!! ================================================================================================================================
!! SUBROUTINE   : littercalc
!!
!!\BRIEF        Calculation of the litter decomposition and therefore of the 
!! heterotrophic respiration from litter following Parton et al. (1987).
!!
!! DESCRIPTION  : The littercal routine splits the litter in 4 pools: 
!! aboveground metaboblic, aboveground structural, belowground metabolic and 
!! belowground structural. the fraction (F) of plant material going to metabolic 
!! and structural is defined following Parton et al. (1987)
!! \latexonly
!! \input{littercalc1.tex}
!! \endlatexonly
!! \n
!! where L is the lignin content of the plant carbon pools considered and CN 
!! its CN ratio. L and CN are fixed parameters for each plant carbon pools,
!! therefore it is the ratio between each plant carbon pool within a PFT, which
!! controlled the part of the total litter, that will be considered as
!! recalcitrant (i.e. structural litter) or labile (i.e. metabolic litter).\n  
!! 
!! The routine calculates the fraction of aboveground litter which is metabolic
!! or structural (the litterpart variable) which is then used in lpj_fire.f90.\n 
!! 
!! In the section 2, the routine calculate the new plant material entering the
!! litter pools by phenological death of plants organs (corresponding to the
!! variable turnover) and by fire, herbivory and others non phenological causes
!! (variable bm_to_litter). This calculation is first done for each PFT and then
!! the values calculated for each PFT are added up. Following the same approach
!! the lignin content of the total structural litter is calculated and will be
!! then used as a factor control of the decomposition of the structural litter
!! (lignin_struc) in the section 5.1.2. A test is performed to avoid that we add
!! more lignin than structural litter. Finally, the variable litterpart is
!! updated.\n
!! 
!! In the section 3 and 4 the temperature and the moisture controlling the
!! decomposition are calculated for above and belowground. For aboveground
!! litter, air temperature and litter moisture are calculated in sechiba and used 
!! directly. For belowground, soil temperature and moisture are also calculated 
!! in sechiba but are modulated as a function of the soil depth. The modulation 
!! is a multiplying factor exponentially distributed between 0 (in depth) and 1
!! in surface.\n  
!! 
!! Then, in the section 5, the routine calculates the structural litter decomposition 
!! (C) following first order kinetics following Parton et al. (1987).
!! \latexonly
!! \input{littercalc2.tex}
!! \endlatexonly
!! \n
!! with k the decomposition rate of the structural litter. 
!! k corresponds to
!! \latexonly
!! \input{littercalc3.tex}
!! \endlatexonly
!! \n
!! with littertau the turnover rate, T a function of the temperature and M a function of
!! the moisture described below.\n
!!  
!! Then, the fraction of dead leaves (DL) composed by aboveground structural litter is
!! calculated as following
!! \latexonly
!! \input{littercalc4.tex}
!! \endlatexonly
!! \n
!! with k the decomposition rate of the structural litter previously
!! described.\n
!!
!! In the section 5.1, the fraction of decomposed structural litter
!! incorporated to the soil (Input) and its associated heterotrophic respiration are
!! calculated. For structural litter, the C decomposed could go in the active
!! soil carbon pool or in the slow carbon, as described in 
!! stomate_soilcarbon.f90.\n
!! \latexonly
!! \input{littercalc5.tex}
!! \endlatexonly
!! \n
!! with f a parameter describing the fraction of structural litter incorporated
!! into the considered soil carbon pool, C the amount of litter decomposed and L 
!! the amount of lignin in the litter. The litter decomposed which is not
!! incorporated into the soil is respired.\n
!!
!! In the section 5.2, the fraction of decomposed metabolic litter
!! incorporated to the soil and its associated heterotrophic respiration are
!! calculated with the same approaches presented for 5.1 but no control factor
!! depending on the lignin content are used.\n
!! 
!! In the section 6 the dead leaf cover is calculated through a call to the 
!! deadleaf subroutine presented below.\n
!!
!! In the section 7, if the flag SPINUP_ANALYTIC is set to true, we fill MatrixA
!! and VectorB following Lardy(2011).
!!
!! MAIN OUTPUT VARIABLES: ::deadleaf_cover, ::resp_hetero_litter, ::soilcarbon_input, 
!! ::control_temp, ::control_moist
!!
!! REFERENCES:
!! - Parton, WJ, Schimel, DS, Cole, CV, and Ojima, DS. 1987. Analysis
!! of factors controlling soil organic matter levels in Great Plains
!! grasslands. Soil Science Society of America journal (USA)
!! (51):1173-1179.
!! - Lardy, R, et al., A new method to determine soil organic carbon equilibrium,
!! Environmental Modelling & Software (2011), doi:10.1016|j.envsoft.2011.05.016
!!
!! FLOWCHART    :
!! \latexonly
!! \includegraphics(scale=0.5){littercalcflow.jpg}
!! \endlatexonly
!! \n
!_ ================================================================================================================================

  SUBROUTINE littercalc (npts, turnover, bm_to_litter, &
       veget_cov_max, tsurf, tsoil, soilhum, litterhum, som, clay, silt, & 
       soil_n_min, litter, dead_leaves, lignin_struc, &
       lignin_wood, n_mineralisation, deadleaf_cover, resp_hetero_litter, &
       som_input, control_temp, control_moist, n_fungivores, &
       MatrixA, VectorB, CN_target, CN_som_litter_longterm, tau_CN_longterm, &
       ld_redistribute,circ_class_biomass,circ_class_n)

    !! 0. Variable and parameter declaration
    
    !! 0.1 Input variables

    INTEGER(i_std), INTENT(in)                                  :: npts               !! Domain size - number of grid pixels
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements), INTENT(in) :: turnover         !! Turnover rates of plant biomass 
                                                                                      !! @tex $(gC m^{-2} dt\_slow^{-1})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements), INTENT(in) :: bm_to_litter     !! Conversion of biomass to litter 
                                                                                      !! @tex $(gC m^{-2} dt\_slow^{-1})$ @endtex 
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in)                  :: veget_cov_max      !! PFT "Maximal" coverage fraction of a PFT 
                                                                                      !! defined in the input vegetation map 
                                                                                      !! @tex $(m^2 m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(npts), INTENT(in)                    :: tsurf              !! Temperature (K) at the surface
    REAL(r_std), DIMENSION(npts,nbdl), INTENT(in)               :: tsoil              !! Soil temperature (K)
    REAL(r_std), DIMENSION(npts,nbdl), INTENT(in)               :: soilhum            !! Daily soil humidity of each soil layer 
                                                                                      !! (unitless)
    REAL(r_std), DIMENSION(npts), INTENT(in)                    :: litterhum          !! Daily litter humidity (unitless)
    REAL(r_std), DIMENSION(npts), INTENT(in)                    :: clay               !! Clay fraction (unitless, 0-1)
    REAL(r_std), DIMENSION(npts), INTENT(in)                    :: silt               !! Silt fraction (unitless, 0-1)
    REAL(r_std), DIMENSION(npts,nvm,nnspec),INTENT(inout)       :: soil_n_min         !! mineral nitrogen in the soil (gN/m**2)  
    REAL(r_std), DIMENSION(npts,nvm,ncirc,nparts,nelements), INTENT(in) :: circ_class_biomass !! Biomass components of the model tree  
                                                                                      !! within a circumference class 
                                                                                      !! @tex $(g C ind^{=1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,ncirc), INTENT(in)          :: circ_class_n       !! Number of individuals in each circ class
                                                                                        
    !! 0.2 Output variables
    
    REAL(r_std), DIMENSION(npts), INTENT(out)                   :: deadleaf_cover     !! Fraction of soil covered by dead leaves 
                                                                                      !! over all PFTs (0-1, unitless)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)               :: resp_hetero_litter !! Litter heterotrophic respiration. The unit
                                                                                      !! is given by m^2 of ground.  
                                                                                      !! @tex $(gC dt_sechiba one\_day^{-1}) m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(npts,ncarb,nvm,nelements), INTENT(out) :: som_input        !! Quantity of Carbon (or Nitrogen) going into SOM pools
                                                                                      !! from litter decomposition. The unit is  
                                                                                      !! given by m^2 of ground 
                                                                                      !! @tex $(gC(orN) m^{-2} dt\_slow^{-1})$ @endtex 
    REAL(r_std), DIMENSION(npts,nlevs), INTENT(out)             :: control_temp       !! Temperature control of heterotrophic 
                                                                                      !! respiration, above and below (0-1, 
                                                                                      !! unitless)
    REAL(r_std), DIMENSION(npts,nlevs), INTENT(out)             :: control_moist      !! Moisture control of heterotrophic 
                                                                                      !! respiration (0.25-1, unitless)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)               :: n_fungivores       !! Fraction of N released for plant uptake
                                                                                      !! due to fungivore consumption.
    REAL(r_std), DIMENSION(npts,nvm,nbpools,nbpools), INTENT(out) :: MatrixA          !! Matrix containing the fluxes between the
                                                                                      !! carbon pools per sechiba time step 
                                                                                      !! @tex $(gC.m^2.day^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nbpools), INTENT(out)       :: VectorB            !! Vector containing the litter increase per
                                                                                      !! sechiba time step
                                                                                      !! @tex $(gC m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,ncarb), INTENT(out)         :: CN_target          !! C to N ratio of SOM flux from one pool to another (gN m-2 dt-1) 
    REAL(r_std), DIMENSION(nvm), INTENT(out)                    :: ld_redistribute    !! logical set to redistribute som and litter  
 
    !! 0.3 Modified variables
   
    REAL(r_std), DIMENSION(npts,ncarb,nvm,nelements), INTENT(in)          :: som      !! SOM pools: active, slow, or passive, \f$(gC m^{2})$\f 
    REAL(r_std), DIMENSION(npts,nlitt,nvm,nlevs,nelements), INTENT(inout) :: litter   !! Metabolic and structural litter,above and
                                                                                      !! below ground. The unit is given by m^2 of 
                                                                                      !! ground @tex $(gC m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nlitt), INTENT(inout)       :: dead_leaves        !! Dead leaves per ground unit area, per PFT,
                                                                                      !! metabolic and structural in 
                                                                                      !! @tex $(gC m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nlevs), INTENT(inout)       :: lignin_struc       !! Ratio Lignin content in structural litter,
                                                                                      !! above and below ground, (0-1, unitless)
    REAL(r_std), DIMENSION(npts,nvm,nlevs), INTENT(inout)       :: lignin_wood        !! Ratio Lignin/Carbon in woody litter,	
                                                                                      !! above and below ground, (0-1, unitless)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)             :: n_mineralisation   !! Mineral N pool (gN m-2)

    REAL(r_std), DIMENSION(npts,nvm,nbpools), INTENT(inout)     :: CN_som_litter_longterm !! Longterm CN ratio of litter and som pools (gC/gN)
    REAL(r_std), INTENT(inout)                                  :: tau_CN_longterm        !! Counter used for calculating the longterm CN_ratio of som and litter pools [seconds]

    !! 0.4 Local variables
 
    REAL(r_std)                                                 :: dt                 !! Time step of the simulations for stomate
                                                                                      !! @tex $(dt_sechiba one\_day^{-1})$ @endtex 
    REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:,:,:)          :: litterfrac         !! The fraction of leaves, wood, etc. that 
                                                                                      !! goes into metabolic and structural 
                                                                                      !! litterpools (0-1, unitless)
!$OMP THREADPRIVATE(litterfrac)
    REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:)                :: z_soil             !! Soil levels (m)
!$OMP THREADPRIVATE(z_soil)
    REAL(r_std), DIMENSION(npts)                                :: rpc                !! Integration constant for vertical root 
                                                                                      !! profiles (unitless)
    REAL(r_std), SAVE, DIMENSION(nlitt)                         :: litter_dec_fac     !! Turnover time in litter pools (days)
!$OMP THREADPRIVATE(litter_dec_fac)
    REAL(r_std), SAVE, DIMENSION(nlitt,ncarb,nlevs)             :: frac_soil          !! Fraction of litter that goes into soil 
                                                                                      !! (litter -> carbon, above and below). The
                                                                                      !! remaining part goes to the atmosphere
!$OMP THREADPRIVATE(frac_soil)
    REAL(r_std), DIMENSION(npts)                                :: tsoil_decomp       !! Temperature used for decompostition in 
                                                                                      !! soil (K)
    REAL(r_std), DIMENSION(npts)                                :: soilhum_decomp     !! Humidity used for decompostition in soil
                                                                                      !! (unitless)
    REAL(r_std), DIMENSION(npts)                                :: fd                 !! Fraction of structural or metabolic litter
                                                                                      !! decomposed (unitless)
    REAL(r_std), DIMENSION(npts,nelements)                      :: qd                 !! Quantity of structural or metabolic litter
                                                                                      !! decomposed @tex $(gC m^{-2})$ @endtex
                                                                                      !! @tex $(gC m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nlevs)                      :: old_struc          !! Old structural litter, above and below 
                                                                                      !! @tex $(gC m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nlevs)                      :: old_woody          !! Old woody litter, above and below 
                                                                                      !! @tex $(gC m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(npts,nlitt,nvm,nlevs,nelements)      :: litter_inc         !! Increase of metabolic and structural 
                                                                                      !! litter, above and below ground. The unit 
                                                                                      !! is given by m^2 of ground. 
                                                                                      !! @tex $(gC m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nlevs)                      :: lignin_struc_inc   !! Lignin increase in structural litter, 
                                                                                      !! above and below ground. The unit is given 
                                                                                      !! by m^2 of ground. 
                                                                                      !! @tex $(gC m^{-2})$ @endtex                                             
    REAL(r_std), DIMENSION(npts,nvm,nlevs)                      :: lignin_wood_inc    !! Lignin increase in woody litter, 
                                                                                      !! above and below ground. The unit is given 
                                                                                      !! by m^2 of ground. 
                                                                                      !! @tex $(gC m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(npts)                                :: zdiff_min          !! Intermediate field for looking for minimum
                                                                                      !! of what? this is not used in the code. 
                                                                                      !! [??CHECK] could we delete it?
    CHARACTER(LEN=10), DIMENSION(nlitt)                         :: litter_str         !! Messages to write output information about
                                                                                      !! the litter
    CHARACTER(LEN=22), DIMENSION(nparts)                        :: part_str           !! Messages to write output information about
                                                                                      !! the plant
    CHARACTER(LEN=7), DIMENSION(ncarb)                          :: carbon_str         !! Messages to write output information about
                                                                                      !! the soil carbon
    CHARACTER(LEN=5), DIMENSION(nlevs)                          :: level_str          !! Messages to write output information about
                                                                                      !! the level (aboveground or belowground litter)
    INTEGER(i_std)                                              :: ilitt,ilev,ibdl    !! Indices (unitless)
    INTEGER(i_std)                                              :: ipts,ivm, j        !! Indices (unitless)
    INTEGER(i_std)                                              :: icarb,inspec,iele  !! Indices (unitless)
    INTEGER(i_std)                                              :: ipar,imbc,iicarb   !! Indices (unitless)
    REAL(r_std), DIMENSION(npts,nvm)                            :: f_soil_n_min       !! soil_n_min function response used for defing C to N target ratios (-) 
    REAL(r_std), DIMENSION(npts,nvm)                            :: f_pnc              !! plant nitrogen concentration function response
                                                                                      !! used for defining C to N target ratios (-)  
    INTEGER(i_std)                                              :: itarget            !! target som pool 
    REAL(r_std), DIMENSION(npts,nvm,nparts)                     :: CN                 !! CN ratio of the litter pools 
    REAL(r_std), DIMENSION(npts,nelements)                      :: summ  
    INTEGER(i_std)                                              :: ier                !! Error handling
    REAL(r_std), DIMENSION(npts,nvm,nmbcomp,nelements)          :: check_intern       !! Contains the components of the internal
                                                                                      !! mass balance check for this routine
                                                                                      !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements)                  :: closure_intern     !! Check closure of internal mass balance
                                                                                      !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements)                  :: pool_start         !! Start and end pool of this routine 
                                                                                      !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements)                  :: pool_end           !! Start and end pool of this routine 
                                                                                      !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm)                            :: temp
    REAL(r_std), DIMENSION(npts)                                :: err                     !! Array for error checking 
    REAL(r_std), DIMENSION(ncarb,nelements)                     :: som_input_old           !! Used to store som_input values
                                                                                           !! in case of inadequate soil N
    REAL(r_std), DIMENSION(ncarb,nelements)                     :: delta_som_input         !! Used to store som_input values
                                                                                           !! in case of inadequate soil N
    REAL(r_std), DIMENSION(nvm,nelements)                       :: som_input_new           !! Used to recalculate som_input in
                                                                                           !! case of inadequate soil N
    REAL(r_std)                                                 :: CN_input_total          !! overall C/N ratio of som_input_total
    REAL(r_std), DIMENSION(nvm,nelements)                       :: som_input_total         !! Used to recalculate som_input in
                                                                                           !! case of inadequate soil N
    REAL(r_std), DIMENSION(nelements)                           :: litter_total_new        !! Updated litter pool in case of nitrogen
                                                                                           !! deficit
    REAL(r_std), DIMENSION(npts,nvm)                            :: total_init_nitrogen     !! Litter pool at the very start of the 
                                                                                           !! calculations. Required to correct for the
                                                                                           !! nitrogen deficit
    REAL(r_std)                                                 :: n_mineralisation_init   !! temp variable in case of overspending
    REAL(r_std)                                                 :: delta_hetero_litter     !! reduction factor of resp_hetero
                                                                                           !! in case of nitrogen shortage
    
!!$    REAL(r_std), DIMENSION(npts,nvm,nelements)                  :: delta_s_in              !! som_input removed
    REAL(r_std), DIMENSION(npts,nvm,nelements)                  :: share_litter_struc_a    !! Fractions of litter in various pools
    REAL(r_std), DIMENSION(npts,nvm,nelements)                  :: share_litter_struc_b    !! Fractions of litter in various pools
    REAL(r_std), DIMENSION(npts,nvm,nelements)                  :: share_litter_met_a      !! Fractions of litter in various pools
    REAL(r_std), DIMENSION(npts,nvm,nelements)                  :: share_litter_met_b      !! Fractions of litter in various pools
    REAL(r_std), DIMENSION(npts,nvm,nelements)                  :: share_litter_woody_a    !! Fractions of litter in various pools
    REAL(r_std), DIMENSION(npts,nvm,nelements)                  :: share_litter_woody_b    !! Fractions of litter in various pools
    REAL(r_std)                                                 :: share_som_input_active  !! Active pool fraction of 
                                                                                           !! som_input(:,:,ivm,initrogen)
    REAL(r_std)                                                 :: share_som_input_slow    !! Slow pool fraction of
                                                                                           !! som_input(:,:,ivm,initrogen)
    REAL(r_std)                                                 :: share_som_input_surface !! Surface pool fraction of
                                                                                           !! som_input(:,:,ivm,initrogen)
    REAL(r_std), DIMENSION(npts,ncarb,nvm,nlevs)                :: N_deficit               !! Missing litter N (per pool) after pulling 
                                                                                           !! additional litter N to satisfy 
                                                                                           !! N_mineralisation demand (5.5)
    REAL(r_std)                                                 :: N_deficit_total         !! Total litter N deficit (5.5)
    REAL(r_std), DIMENSION(npts,nvm)                            :: litter_max              !! Finds largest current litter pool
    REAL(r_std), DIMENSION(npts,nvm,ncarb,ncarb,nelements)      :: flux                    !! Fluxes between carbon pools (gC/m^2)
    REAL(r_std), DIMENSION(npts,nvm,ncarb,nelements)            :: fluxtot                 !! Total flux out of carbon pools (gC/m^2)
    REAL(r_std), DIMENSION(npts,ncarb,nvm,nelements)            :: som_temp                !! Temp variable
    REAL(r_std), DIMENSION(npts,ncarb,ncarb)                    :: frac_carb               !! Flux fractions between carbon pools
    REAL(r_std), DIMENSION(npts,ncarb)                          :: frac_resp               !! Flux fractions from carbon pools to the atmosphere
                                                                                           !! atmosphere (respiration) (unitless, 0-1)
    REAL(r_std), DIMENSION(ncarb)                               :: som_turn                !! Residence time in SOM pools (days)
    REAL(r_std), DIMENSION(npts,nvm)                            :: n_mineralisation_som    !! n_mineralisation that we will calculate in the 
                                                                                           !! next routine
    REAL(r_std), DIMENSION(npts,nvm)                            :: n_min_new               !! Temp variable
    REAL(r_std), DIMENSION(npts,nvm)                            :: n_min_old               !! Temp variable
    REAL(r_std), DIMENSION(nvm,nelements)                       :: som_input_total_new     !! Truncated value of N moved from 
                                                                                           !! n_mineralisation to som_input
    REAL(r_std), DIMENSION(npts,nvm)                            :: share_som_input_act_n   !! Active pool fraction of 
                                                                                           !! som_input(:,:,ivm,initrogen)
    REAL(r_std), DIMENSION(npts,nvm)                            :: share_som_input_slow_n  !! Slow pool fraction of
                                                                                           !! som_input(:,:,ivm,initrogen)
    REAL(r_std), DIMENSION(npts,nvm)                            :: share_som_input_surf_n  !! Surface pool fraction of
                                                                                           !! som_input(:,:,ivm,initrogen)
    REAL(r_std), DIMENSION(npts,nvm)                            :: resp_hetero_soil        !! Soil heterotrophic respiration (gC/m^2/day)   
    REAL(r_std), DIMENSION(npts,nvm,nelements)                  :: total_som_n_flux        !! Total flux between all soil pools. Used to adjust 
                                                                                           !! som if overspending @tex $(N gN m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm)                            :: f_fungivores            !! Fraction of decomposed litter consumed by 
                                                                                           !! fungivores
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements)           :: biomass_temp            !! Temporal value to calculate fungivores 

!_ ================================================================================================================================

    IF (printlev>=3) WRITE(numout,*) 'Entering littercalc'

    !! 1.1 Initialize local printlev
    !printlev_loc=get_printlev('litter')


  !! 1. Initialisations of the different fields during the first call of the routine 
    dt = dt_sechiba/one_day

    IF ( firstcall_litter ) THEN

       IF ( .NOT. ALLOCATED(litterfrac) ) THEN
          ALLOCATE(litterfrac(npts,nvm,nparts,nlitt))
       ENDIF

       !! 1.1.2 residence times in litter pools (days)
       ! .5 years, 3 years and ??30 years??(2.45)
       litter_dec_fac(imetabolic) = turn_metabolic / one_year      
       litter_dec_fac(istructural) = turn_struct / one_year        
       litter_dec_fac(iwoody) = turn_woody / one_year  
      
       !! 1.1.5 decomposition flux fraction that goes into soil
       !  (litter -> carbon, above and below)
       !  1-frac_soil goes into atmosphere
       frac_soil(:,:,:) = zero

       ! structural litter: lignin fraction goes into slow pool + respiration,
       ! rest into active pool + respiration
       frac_soil(istructural,isurface,iabove) = frac_soil_struct_sua
       frac_soil(istructural,iactive,ibelow) = frac_soil_struct_ab
       frac_soil(istructural,islow,iabove) = frac_soil_struct_sa
       frac_soil(istructural,islow,ibelow) = frac_soil_struct_sb

       ! metabolic litter: all goes into active pool + respiration.
       ! Nothing into slow or passive pool.
       frac_soil(imetabolic,isurface,iabove) = frac_soil_metab_sua
       frac_soil(imetabolic,iactive,ibelow) = frac_soil_metab_ab

       ! woody litter: lignin fraction goes into slow pool + respiration,
       ! rest into active pool + respiration. Same as strucutural
       !!$ frac_soil(iwoody,isurface,iabove) = frac_soil_struct_sua
       !!$ frac_soil(iwoody,iactive,ibelow) = frac_soil_struct_ab
       !!$ frac_soil(iwoody,islow,iabove) = frac_soil_struct_sa
       !!$ frac_soil(iwoody,islow,ibelow) = frac_soil_struct_sb

       ! woody litter lignin fraction should be lower than that of structural
       ! litter, else woody litter will not decompose quickly enough.
       ! In effect, this decreases the carbon use efficiency of the system to  
       ! allow for faster litter decomposition. The parameter frac_woody is not
       ! well-constrained and was tested for a boreal PFT in northern Sweden.       
       frac_soil(iwoody,isurface,iabove) = frac_woody * frac_soil_struct_sua
       frac_soil(iwoody,iactive,ibelow) = frac_woody * frac_soil_struct_ab
       frac_soil(iwoody,islow,iabove) = frac_woody * frac_soil_struct_sa
       frac_soil(iwoody,islow,ibelow) = frac_woody * frac_soil_struct_sb

       !! 1.2 soil levels
       ALLOCATE(z_soil(0:nbdl), stat=ier)
       IF ( ier /= 0 ) CALL ipslerr(3,'littercalc','Pb in allocate of z_soil','','')

       z_soil(0) = zero
       z_soil(1:nbdl) = diaglev(1:nbdl)

       !! 1.3 messages
       litter_str(imetabolic) = 'metabolic'
       litter_str(istructural) = 'structural'
       litter_str(iwoody) = 'woody'

       carbon_str(iactive) = 'active'
       carbon_str(isurface) = 'surface'
       carbon_str(islow) = 'slow'
       carbon_str(ipassive) = 'passive'

       level_str(iabove) = 'above'
       level_str(ibelow) = 'below'

       part_str(ileaf) = 'leaves'
       part_str(isapabove) = 'sap above ground'
       part_str(isapbelow) = 'sap below ground'
       part_str(iheartabove) = 'heartwood above ground'
       part_str(iheartbelow) = 'heartwood below ground'
       part_str(iroot) = 'roots'
       part_str(ifruit) = 'fruits'
       part_str(icarbres) = 'carbohydrate reserve'
       part_str(ilabile) = 'labile reserve'

       ! Debug
       IF (printlev_loc.GE.4) THEN
          WRITE(numout,*) 'litter:'

          WRITE(numout,*) '   > C/N ratios: '
          DO ipar = 1, nparts
             WRITE(numout,*) '       ', part_str(ipar), ': ',CN_fix(ipar)
          ENDDO
          
          WRITE(numout,*) '   > scaling depth for decomposition (m): ',z_decomp

          WRITE(numout,*) '   > minimal carbon residence time in litter pools (d):'
          DO ilitt = 1, nlitt
             WRITE(numout,*) '       ',litter_str(ilitt),':',litter_dec_fac(ilitt)
          ENDDO
          
          WRITE(numout,*) '   > litter decomposition flux fraction that really goes '
          WRITE(numout,*) '     into carbon pools (rest into the atmosphere):'
          DO ilitt = 1, nlitt
             DO ilev = 1, nlevs
                DO icarb = 1, ncarb
                   WRITE(numout,*) '       ',litter_str(ilitt),' ',level_str(ilev),' -> ',&
                        carbon_str(icarb),':', frac_soil(ilitt,icarb,ilev)
                ENDDO
             ENDDO
          ENDDO

       ENDIF
       !-
          
       firstcall_litter = .FALSE.

    ENDIF

    !! 1.1.3 litter fractions:
    !  What fraction of leaves, wood, etc. goes into metabolic and 
    !  structural litterpools

    ! PFT 1 needs to be initialised for when everything is printed below
    litterfrac(:,1,:,:)=zero
    CN(:,:,:) = zero

    !+++CHECK+++
    ! Add documentation
    ! Calculate the CN ratio. If the CN ratio cannot be calculated
    ! then use the prescribed value
    DO ipar = 1, nparts
       
       ! Debug
       IF(printlev_loc >=4) THEN
          WRITE(numout,*) 'bm_to_litter carbon for ',&
            part_str(ipar),':',bm_to_litter(:,:,ipar,icarbon)
          WRITE(numout,*) 'bm_to_litter nitrogen for ',&
            part_str(ipar),':',bm_to_litter(:,:,ipar,initrogen)
          WRITE(numout,*) 'turnover carbon for ',&
            part_str(ipar),':',turnover(:,:,ipar,icarbon)
          WRITE(numout,*) 'turnover nitrogen for ',&
            part_str(ipar),':',turnover(:,:,ipar,initrogen)       
       ENDIF
       !-

       WHERE((bm_to_litter(:,:,ipar,initrogen)+&
            turnover(:,:,ipar,initrogen)).GT.min_stomate) 
          CN(:,:,ipar) = (bm_to_litter(:,:,ipar,icarbon)+&
               turnover(:,:,ipar,icarbon))/ & 
               (bm_to_litter(:,:,ipar,initrogen)+&
               turnover(:,:,ipar,initrogen))

       ELSEWHERE

          CN(:,:,ipar) = CN_fix(ipar)

       ENDWHERE

       DO ivm = 1,nvm

          IF ( ((ipar == isapabove) .OR. (ipar == isapbelow) .OR. &
               (ipar == iheartabove) .OR. (ipar == iheartbelow)) .AND. &
               ivm == 1) THEN

             WHERE (SUM(litter(:,iwoody,ivm,:,icarbon),2).GT.min_stomate)

                ! PFT 1 with woody litter will be treated as
                ! a forest
                litterfrac(:,ivm,ipar,iwoody) = 1.0
                litterfrac(:,ivm,ipar,imetabolic) = zero
                litterfrac(:,ivm,ipar,istructural) = zero
                
             ENDWHERE
 
          ELSEIF ( ((ipar == isapabove) .OR. (ipar == isapbelow) .OR. &
               (ipar == iheartabove) .OR. (ipar == iheartbelow)) &
               .AND. is_tree(ivm) ) THEN
             
             ! Woody components of forest PFTs
             litterfrac(:,ivm,ipar,iwoody) = 1.0
             litterfrac(:,ivm,ipar,imetabolic) = zero
             litterfrac(:,ivm,ipar,istructural) = zero

          ELSE 

             ! PFT 1 without woody litter and all non-forest PFTs
             IF( (ipar == icarbres) .OR. (ipar == ilabile)) THEN

                litterfrac(:,ivm,ipar,imetabolic) = 1.0

             ELSE

                litterfrac(:,ivm,ipar,imetabolic) = &
                     MAX(metabolic_ref_frac - metabolic_LN_ratio  * &
                     LC(ivm,ipar) * CN(:,ivm,ipar),zero)

             ENDIF

             litterfrac(:,ivm,ipar,istructural) = &
                  1. - litterfrac(:,ivm,ipar,imetabolic)
             litterfrac(:,ivm,ipar,iwoody) = zero
             
          ENDIF
   
       END DO
       
    END DO

    ! Error checking
    IF(err_act.GT.1)THEN
       DO ipts = 1,npts
          DO ivm = 1,nvm
             DO ipar = 1, nparts
                IF((bm_to_litter(ipts,ivm,ipar,initrogen)+turnover(ipts,ivm,ipar,initrogen)).GT.min_stomate)THEN
                   IF (abs(SUM(litterfrac(ipts,ivm,ipar,:))-1.0).GT.min_stomate) THEN
                      WRITE(numout,*) 'litterfrac - should equal to 1, ', ivm, ipar, &
                           SUM(litterfrac(ipts,ivm,ipar,:))
                      WRITE(numout,*) 'CN, ', CN(ipts,ivm,ipar)
                      WRITE(numout,*) 'CN_target (ncarb), ',CN_target(ipts,ivm,:)
                      CALL ipslerr(3,'stomate_litter',&
                           'litterfrac do not add up to 1','','')
                   ENDIF
                ENDIF
             ENDDO
          ENDDO
       ENDDO
    ENDIF

    ! Error checking
    ! soil_n_min should never be zero. It is a nitrogen pool
    ! expressed in gN m-2
    temp(:,:)=zero
    WHERE(soil_n_min(:,:,iammonium).LT.zero)
       temp(:,:) = un
    ENDWHERE

    IF (SUM(SUM(temp(:,:),2)).GT.zero) THEN
       WRITE(numout,*)'ERROR: soil_n_min iammonium is negative',&
            soil_n_min(:,:,iammonium)     
       CALL ipslerr(3,'stomate_litter',&
            'At least one soil_n_min value',&
            'is negative in stomate_litter','')
    ENDIF
    
    temp(:,:)=zero
    WHERE(soil_n_min(:,:,initrate).LT.zero)
       temp(:,:) = un
    ENDWHERE
    
    IF (SUM(SUM(temp(:,:),2)).GT.zero) THEN
       WRITE(numout,*)'ERROR: soil_n_min initrate is negative', &
            soil_n_min(:,:,initrate)     
       CALL ipslerr(3,'stomate_litter',&
            'At least one soil_n_min value',&
            'is negative in stomate_litter','')
    ENDIF

    !+++HACK+++
    ! This code is OK only when the land cover change and the age
    ! class distribution code are working properly. Better to test
    ! it in those subroutines than to enforce it here. If there are
    ! problems with land cover change or age class distribution the
    ! mass balance for soil_n_min may be violated here.
    DO ipts = 1, npts
       DO ivm = 1, nvm
          DO inspec = 1, nnspec
             IF((veget_cov_max(ipts,ivm).LT.min_stomate) .AND. &
                  (soil_n_min(ipts,ivm,inspec).GT. min_stomate)) THEN
                !WRITE(numout,*) 'PFT,',ivm,', pixel:', ipts,', truc:',inspec
                !WRITE(numout,*) 'veget_cov_max,',veget_cov_max(ipts,ivm)
                !WRITE(numout,*) 'soil_n_min, ',soil_n_min(ipts,ivm,inspec)
                CALL ipslerr(3,'stomate_litter',&
                     'soil_n_min present in pixels without veget_cov_max',&
                     '','')   
             ENDIF
          ENDDO
       ENDDO
    ENDDO
    !++++++++++

    !+++CHECK+++
    ! ADD documentation. What justifies the use of 2? Should 
    ! that parameter be externalized? What does that parameter
    ! represent? The MIN does NOT protect against negative
    ! soil_n_values. Negative values should not occur!
    f_soil_n_min(:,:) = &
         MIN((soil_n_min(:,:,iammonium) + soil_n_min(:,:,initrate)), 2.)
    !+++++++++++

    ! C to N target ratios of different pools
    CN_target(:,:,:)=zero

    ! CN ratio ranges between 3 and 15 for active pool
    ! Figure 4 of Parton et al. (1993) show lower CN values for active 
    ! pool of 2 rather than 3
    CN_target(:,:,iactive)= CN_target_iactive_ref + &
         CN_target_iactive_Nmin * f_soil_n_min(:,:)

    ! CN ratio ranges between 12 and 20 for slow pool
    CN_target(:,:,islow)= CN_target_islow_ref + &
         CN_target_islow_Nmin * f_soil_n_min(:,:)

    ! CN ratio ranges between 7 and 10 for passive pool
    ! OCN uses a fixed value of 9. (don't know why)
    ! Figure 4 of Parton et al. (1993) show lower CN values 
    ! for passive pool of 3 rather than 7
    CN_target(:,:,ipassive)= CN_target_ipassive_ref + &
         CN_target_ipassive_Nmin * f_soil_n_min(:,:)

    ! Plant nitrogen content (%)
    WHERE(litter(:,imetabolic,:,iabove,icarbon)+&
         litter(:,istructural,:,iabove,icarbon).GT.zero)

       !+++CHECK+++
       ! Why do we multiply with 2?, why do we take the
       ! minimun between the calculation and 2. Do those
       ! two 2s represent the same parameter? The 2* may
       ! be used to convert carbon biomass to fresh biomass 
       f_pnc(:,:)=MIN((litter(:,imetabolic,:,iabove,initrogen)+&
            litter(:,istructural,:,iabove,initrogen)) / &
            (2.*(litter(:,imetabolic,:,iabove,icarbon)+&
            litter(:,istructural,:,iabove,icarbon))) * 100., 2.)
       !+++++++++++

    ELSEWHERE

       f_pnc(:,:)= zero

    ENDWHERE    

    !+++CHECK+++
    ! Add documentation
    CN_target(:,:,isurface) = CN_target_isurface_ref + &
         CN_target_isurface_pnc * f_pnc(:,:)
    !+++++++++++

    !! 1.4 set output to zero
    deadleaf_cover(:) = zero
    resp_hetero_litter(:,:) = zero
    som_input(:,:,:,:) = zero
    n_fungivores(:,:) = zero
    f_fungivores(:,:) = zero

    !! 1.5 Initialize check for mass balance closure
    !  The mass balance is calculated at the end of this routine
    !  in section 6. Initial biomass and harvest pool all other
    !  relevant pools were just set to zero.
    IF (err_act.GT.1) THEN

          pool_start(:,:,:) = zero
          pool_start(:,:,initrogen) = pool_start(:,:,initrogen) + &
               n_mineralisation(:,:) * veget_cov_max(:,:)              
          pool_start(:,:,initrogen) = pool_start(:,:,initrogen) + &
               n_fungivores(:,:) * veget_cov_max(:,:)
     
          DO iele = 1,nelements 
             ! The litter pool
             DO ilitt = 1,nlitt
                DO ilev = 1,nlevs
                   pool_start(:,:,iele) = pool_start(:,:,iele) + &
                        (litter(:,ilitt,:,ilev,iele) * veget_cov_max(:,:))
                ENDDO
             ENDDO
             
             ! Pools added to the litter
             DO ipar = 1,nparts
                pool_start(:,:,iele) = pool_start(:,:,iele) + &
                     (turnover(:,:,ipar,iele) + &
                     bm_to_litter(:,:,ipar,iele)) * veget_cov_max(:,:)
             ENDDO
 
          ENDDO

    ENDIF ! err_act.GT.1

    ! Store the initial available N. In case there is overspending of nitrogen
    ! this value can be used to adjust the spending
    WHERE (veget_cov_max(:,:).GT.zero)
       total_init_nitrogen(:,:) = n_mineralisation(:,:) +  &
            SUM(SUM(litter(:,:,:,:,initrogen),4),2) + SUM(turnover(:,:,:,initrogen),3) + &
            SUM(bm_to_litter(:,:,:,initrogen),3)
    ELSEWHERE
       total_init_nitrogen(:,:) = zero
    ENDWHERE

   ! fungivores was applied to avoid the problem that soil mineral nitrogen is used up by
   ! the decomposition after all individuals death. A certain portion (f_fungivores) of nitrogen
   ! pool from the litter was fed to the vegetation using n_fungivores. But not to give extra
   ! nitrogen even in optimum condition, dynamic f_fungivores is more reasonable
   ! than fixed f_fungivores. Before calculating n_fungivores f_fungivoes is
   ! calculated using total biomass and total litter of carbon.  
   ! Underlying idea is to use predator prey relationship between fungi and
   ! fungivores. When there is a lot of litter, there will be a lot of fungi and
   ! so we expect a lot of fungivores. Nitrogen excreted by the fungivores will
   ! be used by plants. This implementation would benefit from a more solid
   ! process understanding and more literature on the topic.
   biomass_temp(:,:,:,:) = cc_to_biomass(npts,nvm,circ_class_biomass(:,:,:,:,:),&
       circ_class_n(:,:,:))

   WHERE ((SUM(biomass_temp(:,:,:,icarbon),3) + SUM(SUM(litter(:,:,:,:,icarbon),4),2) + &
      SUM(bm_to_litter(:,:,:,icarbon),3) ) .LT. min_stomate)
      f_fungivores(:,:) = zero
   ELSEWHERE
      f_fungivores(:,:) = (SUM(SUM(litter(:,:,:,:,icarbon),4),2) + &
       SUM(bm_to_litter(:,:,:,icarbon),3))/(SUM(biomass_temp(:,:,:,icarbon),3) + &
       SUM(SUM(litter(:,:,:,:,icarbon),4),2) + SUM(bm_to_litter(:,:,:,icarbon),3) )
   ENDWHERE
 
  !! 2. Add biomass to different litterpools (per m^2 of ground)
    
    !! 2.1 first, save old structural litter (needed for lignin fractions).
    !     above/below
    DO ilev = 1, nlevs !Loop over litter levels (above and below ground)

       DO ivm = 1,nvm !Loop over PFTs

          old_struc(:,ivm,ilev) = litter(:,istructural,ivm,ilev,icarbon)
          old_woody(:,ivm,ilev) = litter(:,iwoody,ivm,ilev,icarbon)

       ENDDO

    ENDDO
    
    !! 2.2 update litter, dead leaves, and lignin content in structural litter
    litter_inc(:,:,:,:,:) = zero
    lignin_struc_inc(:,:,:) = zero
    lignin_wood_inc(:,:,:) = zero

    DO ivm = 1,nvm !Loop over PFTs

       !! 2.2.1 litter
       DO ilitt = 1, nlitt    !Loop over litter pools (metabolic and structural)
          
          DO iele = 1, nelements  ! Loop over element pools (carbon and nitrogen)
             
             !! 2.2.2 calculate litter increase (per m^2 of ground).
             !  Only a given fracion of fruit turnover is directly 
             !  coverted into litter. Litter increase for each PFT, 
             !  structural and metabolic, above/below
             litter_inc(:,ilitt,ivm,iabove,iele) = & 
                  litterfrac(:,ivm,ileaf,ilitt) * bm_to_litter(:,ivm,ileaf,iele) + & 
                  litterfrac(:,ivm,isapabove,ilitt) * bm_to_litter(:,ivm,isapabove,iele) + & 
                  litterfrac(:,ivm,iheartabove,ilitt) * bm_to_litter(:,ivm,iheartabove,iele) + & 
                  litterfrac(:,ivm,ifruit,ilitt) * bm_to_litter(:,ivm,ifruit,iele) + & 
                  litterfrac(:,ivm,icarbres,ilitt) * bm_to_litter(:,ivm,icarbres,iele) + & 
                  litterfrac(:,ivm,ilabile,ilitt) * bm_to_litter(:,ivm,ilabile,iele) + & 
                  litterfrac(:,ivm,ileaf,ilitt) * turnover(:,ivm,ileaf,iele) + & 
                  litterfrac(:,ivm,isapabove,ilitt) * turnover(:,ivm,isapabove,iele) + & 
                  litterfrac(:,ivm,iheartabove,ilitt) * turnover(:,ivm,iheartabove,iele) + & 
                  litterfrac(:,ivm,ifruit,ilitt) * turnover(:,ivm,ifruit,iele) + & 
                  litterfrac(:,ivm,icarbres,ilitt) * turnover(:,ivm,icarbres,iele)+ &  
                  litterfrac(:,ivm,ilabile,ilitt) * turnover(:,ivm,ilabile,iele) 

             litter_inc(:,ilitt,ivm,ibelow,iele) = & 
                  litterfrac(:,ivm,isapbelow,ilitt) * bm_to_litter(:,ivm,isapbelow,iele) + & 
                  litterfrac(:,ivm,iheartbelow,ilitt) * bm_to_litter(:,ivm,iheartbelow,iele) + & 
                  litterfrac(:,ivm,iroot,ilitt) * bm_to_litter(:,ivm,iroot,iele) + & 
                  litterfrac(:,ivm,isapbelow,ilitt) * turnover(:,ivm,isapbelow,iele) + & 
                  litterfrac(:,ivm,iheartbelow,ilitt) * turnover(:,ivm,iheartbelow,iele) + & 
                  litterfrac(:,ivm,iroot,ilitt) * turnover(:,ivm,iroot,iele) 
             
          ENDDO

          IF ( ilitt /= iwoody ) THEN

             !! 2.2.3 dead leaves, for soil cover.
             dead_leaves(:,ivm,ilitt) = &
                  dead_leaves(:,ivm,ilitt) + litterfrac(:,ivm,ileaf,ilitt) * &
                  ( bm_to_litter(:,ivm,ileaf,icarbon) + &
                  turnover(:,ivm,ileaf,icarbon) )

          ENDIF

          IF(is_tree(ivm))THEN

             IF ( ilitt == istructural ) THEN

                !! 2.2.4 lignin increase in structural litter
                lignin_struc_inc(:,ivm,iabove) = &
                     LC(ivm,ileaf) * bm_to_litter(:,ivm,ileaf,icarbon) + &
                     LC(ivm,ifruit) * bm_to_litter(:,ivm,ifruit,icarbon) + &
                     LC(ivm,icarbres) * bm_to_litter(:,ivm,icarbres,icarbon) + &
                     LC(ivm,ilabile) * bm_to_litter(:,ivm,ilabile,icarbon) + &
                     LC(ivm,ileaf) * turnover(:,ivm,ileaf,icarbon) + &
                     LC(ivm,ifruit) * turnover(:,ivm,ifruit,icarbon) + &
                     LC(ivm,icarbres) * turnover(:,ivm,icarbres,icarbon) + &
                     LC(ivm,ilabile) * turnover(:,ivm,ilabile,icarbon)
             
                lignin_struc_inc(:,ivm,ibelow) = &
                     LC(ivm,iroot) * bm_to_litter(:,ivm,iroot,icarbon) + &
                     LC(ivm,iroot)*turnover(:,ivm,iroot,icarbon)
              
             ELSEIF ( ilitt == iwoody) THEN       
              
                !! 2.2.4 lignin increase in woody litter
                lignin_wood_inc(:,ivm,iabove) = &
                     LC(ivm,isapabove) * bm_to_litter(:,ivm,isapabove,icarbon) + &
                     LC(ivm,iheartabove) * bm_to_litter(:,ivm,iheartabove,icarbon) + &
                     LC(ivm,isapabove) * turnover(:,ivm,isapabove,icarbon) + &
                     LC(ivm,iheartabove) * turnover(:,ivm,iheartabove,icarbon) 
             
                lignin_wood_inc(:,ivm,ibelow) = &
                     LC(ivm,isapbelow) * bm_to_litter(:,ivm,isapbelow,icarbon) + &
                     LC(ivm,iheartbelow) * bm_to_litter(:,ivm,iheartbelow,icarbon) + &
                     LC(ivm,isapbelow)*turnover(:,ivm,isapbelow,icarbon) + &
                     LC(ivm,iheartbelow)*turnover(:,ivm,iheartbelow,icarbon) 
             
             ENDIF ! istructural or iwoody

          ELSE 

             ! None woody vegetation

             IF (ilitt == istructural) THEN

                !! 2.2.4 lignin increase in structural litter
                lignin_struc_inc(:,ivm,iabove) = lignin_struc_inc(:,ivm,iabove) + &
                     LC(ivm,isapabove) * bm_to_litter(:,ivm,isapabove,icarbon) + &
                     LC(ivm,iheartabove) * bm_to_litter(:,ivm,iheartabove,icarbon) + &
                     LC(ivm,isapabove) * turnover(:,ivm,isapabove,icarbon) + &
                     LC(ivm,iheartabove) * turnover(:,ivm,iheartabove,icarbon) 
             
                lignin_struc_inc(:,ivm,ibelow) = lignin_struc_inc(:,ivm,ibelow) + &
                     LC(ivm,isapbelow) * bm_to_litter(:,ivm,isapbelow,icarbon) + &
                     LC(ivm,iheartbelow) * bm_to_litter(:,ivm,iheartbelow,icarbon) + &
                     LC(ivm,isapbelow)*turnover(:,ivm,isapbelow,icarbon) + &
                     LC(ivm,iheartbelow)*turnover(:,ivm,iheartbelow,icarbon)
                
             ENDIF ! istructural
             
          ENDIF ! is_tree
          
       ENDDO ! #nlitt

       ! Error checking
       ! bm_to_litter should be completely transfered to litter_inc
       DO iele = 1,nelements
          IF (ABS(SUM(SUM(SUM(litter_inc(:,:,ivm,:,iele),3),2)) - &
               SUM(SUM(bm_to_litter(:,ivm,:,iele),2)) - SUM(SUM(turnover(:,ivm,:,iele),2))).GT. min_stomate) THEN
             WRITE(numout,*) 'Problem with ivm, element,',ivm,iele
             WRITE(numout,*) 'difference,',ABS(SUM(SUM(SUM(litter_inc(:,:,ivm,:,iele),3),2)) - &
                  SUM(SUM(bm_to_litter(:,ivm,:,iele),2))-SUM(SUM(turnover(:,ivm,:,iele),2)))
             CALL ipslerr(3,'littercalc','problem with litterfrac',&
                  'Not all C/N has been transefered from',&
                  'bm_to_litter & turnover to litter_inc')
          ENDIF
       ENDDO
       
    ENDDO !#nvm

    !! 2.2.5 add new litter (struct/met, above/below)
    litter(:,:,:,:,:) = litter(:,:,:,:,:) + litter_inc(:,:,:,:,:)

    !! 2.2.6 for security: can't add more lignin than structural 
    !! litter (above/below)
    DO ilev = 1, nlevs !Loop over litter levels (above and below ground)

       DO ivm = 1,nvm !Loop over PFTs
          
          lignin_struc_inc(:,ivm,ilev) = MIN( lignin_struc_inc(:,ivm,ilev), &
               litter_inc(:,istructural,ivm,ilev,icarbon) )
          lignin_wood_inc(:,ivm,ilev) = MIN( lignin_wood_inc(:,ivm,ilev), &
               litter_inc(:,iwoody,ivm,ilev,icarbon) )

       ENDDO ! nvm

    ENDDO ! nlevs

    !! 2.2.7 new lignin content: add old lignin and lignin increase, divide by 
    !!       total structural litter (above/below)
    DO ilev = 1, nlevs !Loop over litter levels (above and below ground)
       
       DO ivm = 1,nvm !Loop over PFTs

           WHERE( litter(:,istructural,ivm,ilev,icarbon) .GT. min_stomate )

              lignin_struc(:,ivm,ilev) = ( lignin_struc(:,ivm,ilev) * &
                   old_struc(:,ivm,ilev) + lignin_struc_inc(:,ivm,ilev) ) / &
                   litter(:,istructural,ivm,ilev,icarbon)
              
           ELSEWHERE

              lignin_struc(:,ivm,ilev) = zero
              
           ENDWHERE
     
           WHERE ( litter(:,iwoody,ivm,ilev,icarbon) .GT. min_stomate )

              lignin_wood(:,ivm,ilev) = ( lignin_wood(:,ivm,ilev) * &
                   old_woody(:,ivm,ilev) + lignin_wood_inc(:,ivm,ilev) ) / &
                   litter(:,iwoody,ivm,ilev,icarbon)

           ELSEWHERE

              lignin_wood(:,ivm,ilev) = zero

           ENDWHERE
      
        ENDDO ! ivm
        
     ENDDO ! ilev
    
    
     !! 3. Temperature control on decay: Factor between 0 and 1
     !! 3.1 above: surface temperature
     control_temp(:,iabove) = control_temp_func(npts, tsurf)
     
     !! 3.2 below: convolution of temperature and decomposer profiles
     !!            (exponential decomposer profile supposed)
     !! 3.2.1 rpc is an integration constant such that the integral of 
     !  the root profile is 1.
     rpc(:) = un / ( un - EXP( -z_soil(nbdl) / z_decomp ) )
     
     !! 3.2.2 integrate over the nbdl levels
     tsoil_decomp(:) = zero
     
     DO ibdl = 1, nbdl
        
        tsoil_decomp(:) = &
             tsoil_decomp(:) + tsoil(:,ibdl) * rpc(:) * &
             ( EXP( -z_soil(ibdl-1)/z_decomp ) - EXP( -z_soil(ibdl)/z_decomp ) )

     ENDDO
     IF(printlev_loc.GE.4)THEN
         WRITE(numout,*) 'rpc',rpc(:)
         WRITE(numout,*) 'tsoil',tsoil(:,:)
         WRITE(numout,*) 'z_soil',z_soil(:)
         WRITE(numout,*) 'z_decomp',z_decomp
     ENDIF      
     
     control_temp(:,ibelow) = control_temp_func (npts, tsoil_decomp)
     
     !! 4. Moisture control. Factor between 0 and 1
     !! 4.1 above the ground: litter humidity
     control_moist(:,iabove) = control_moist_func (npts, litterhum)
     
     !! 4.2 below: convolution of humidity and decomposer profiles
     !            (exponential decomposer profile supposed)
     !! 4.2.1 rpc is an integration constant such that the integral 
     ! of the root profile is 1.
     rpc(:) = un / ( un - EXP( -z_soil(nbdl) / z_decomp ) )
     
     !! 4.2.2 integrate over the nbdl levels
     soilhum_decomp(:) = zero
     
     DO ibdl = 1, nbdl !Loop over soil levels
        
        soilhum_decomp(:) = &
             soilhum_decomp(:) + soilhum(:,ibdl) * rpc(:) * &
             ( EXP( -z_soil(ibdl-1)/z_decomp ) - EXP( -z_soil(ibdl)/z_decomp ) )
        
     ENDDO
     
     control_moist(:,ibelow) = control_moist_func (npts, soilhum_decomp)

     ! Debug
     IF(printlev_loc.GE.4) THEN
        WRITE(numout,*) 'lignin_wood, ', test_pft, &
             lignin_wood(:,test_pft,:)
        WRITE(numout,*) 'lignin_struc, ', test_pft, &
             lignin_struc(:,test_pft,:)
        WRITE(numout,*) 'dead_leaves, ', test_pft, &
             dead_leaves(:,test_pft,:)
        WRITE(numout,*) 'control_temp above', control_temp(:,iabove)
        WRITE(numout,*) 'tsurf', tsurf(:)
        WRITE(numout,*) 'control_temp below', control_temp(:,ibelow)
        WRITE(numout,*) 'tsoil_decomp', tsoil_decomp(:)  
        WRITE(numout,*) 'control_moist above ', control_moist(:,iabove)
        WRITE(numout,*) 'literhum', litterhum(:)
        WRITE(numout,*) 'control_moist below ', control_moist(:,ibelow)
        WRITE(numout,*) 'soil_hum_decomp', soilhum_decomp(:)
      
     ENDIF

  !! 5. fluxes from litter to carbon pools and respiration

     DO ilev = 1, nlevs !Loop over litter levels (above and below ground)
        
        DO ivm = 1,nvm !Loop over PFTs
           
           IF (ilev.EQ.iabove)THEN 
              itarget=isurface 
           ELSE 
              itarget=iactive 
           ENDIF

           !! 5.1 structural litter: goes into active and slow carbon 
           !  pools + respiration
           
           !! 5.1.1 total quantity of structural litter which is decomposed
           fd(:) = dt*litter_dec_fac(istructural) * &
                control_temp(:,ilev) * control_moist(:,ilev) * &
                exp( -litter_struct_coef * lignin_struc(:,ivm,ilev) )

           ! Ensure that fd is between 0 and 1
           fd(:) = MAX(zero,MIN(fd(:),un))
           
           !! decompose same fraction of structural part of dead leaves. 
           ! Not exact as lignin content is not the same as that of the total 
           ! structural litter. to avoid a multiple (for ibelow and iabove) 
           ! modification of dead_leaves, we do this test to do this calculate 
           ! only ones in 1,nlev loop
           IF (ilev == iabove)  THEN
              
              dead_leaves(:,ivm,istructural) = &
                   dead_leaves(:,ivm,istructural) * ( un - fd(:) )
              
           ENDIF
           
           !! 5.1.2 Calculate the amount of structural litter
           !  that was decomposed
           err(:) = zero
           WHERE ((litter(:,istructural,ivm,ilev,icarbon).GE.zero) .AND. &
                (litter(:,istructural,ivm,ilev,initrogen).GE.zero))
              
              ! Calculate the quantity of structural litter that should
              ! be decomposed
              qd(:,icarbon) = litter(:,istructural,ivm,ilev,icarbon) * fd(:)
              qd(:,initrogen) = litter(:,istructural,ivm,ilev,initrogen) * fd(:)

           ELSEWHERE

              ! Keep track of how many errors there are
              err(:) = 1.

           ENDWHERE

           ! Check for errors 
           IF (SUM(err(:)) .GT. zero) THEN
              
              WRITE(numout,*) 'Number of errors found: ,', SUM(err(:))
              WRITE(numout,*) 'Negative structural litter pools in stomate_litter'
              WRITE(numout,*) 'That does not make any sense'
              DO iele = 1,nelements
                 WRITE(numout,*) 'Structural litter pool, ', &
                      ivm, ilev, iele, litter(:,istructural,ivm,ilev,iele)
              END DO
              CALL ipslerr(3,'stomate_litter','Negative litter pools',&
                   'This should not be the case','')
              !-
              
           END IF   
             
           ! Debug
           IF(printlev_loc.GE.4 .AND. ivm.EQ.test_pft)THEN
              WRITE(numout,*) 'fd, ',fd(:)
              WRITE(numout,*) 'initial condition, ',ilev
              WRITE(numout,*) 'dead_leaves(istructural), ',&
                   dead_leaves(:,test_pft,istructural)
              WRITE(numout,*) 'fracsoil, ',frac_soil(istructural,itarget,ilev)
              WRITE(numout,*) 'resp_hetero, ',resp_hetero_litter(:,test_pft)
              WRITE(numout,*) 'som_input - C, ', &
                   SUM(som_input(:,:,test_pft,icarbon),2)
              WRITE(numout,*) 'n_mineralisation, ',n_mineralisation(:,test_pft)
              WRITE(numout,*) 'som_input - N, ',&
                   SUM(som_input(:,:,test_pft,initrogen),2)
              WRITE(numout,*) ''
           ENDIF
           !-

           ! Subtract the mineralized fraction from the litter but make sure
           ! the mineralisation cannot become negative
           err(:) = zero   
           WHERE ( (litter(:,istructural,ivm,ilev,icarbon).GE.qd(:,icarbon)) .AND. &
                (litter(:,istructural,ivm,ilev,initrogen).GE.qd(:,initrogen)))

              ! qd is calculated as litter(:,istructural,ivm,ilev,iele)*fd(:)
              ! as long as fd is positive (which was enforced above) the
              ! subtraction below should never result in a number that is
              ! less than zero (except for precision issues). fd is the same
              ! for carbon and nitrogen and therefore the ratio is preserved
              ! during mineralization. The test conditions should be
              ! satisfied for C and N at the same time.
              litter(:,istructural,ivm,ilev,icarbon) = &
                   litter(:,istructural,ivm,ilev,icarbon) - qd(:,icarbon)
              litter(:,istructural,ivm,ilev,initrogen) = &
                   litter(:,istructural,ivm,ilev,initrogen) - qd(:,initrogen)

           ELSEWHERE

              err(:) = 1.

           ENDWHERE

           ! Check for errors 
           IF (SUM(err(:)) .GT. zero) THEN
              
              WRITE(numout,*) 'Number of errors found: ,', SUM(err(:))
              WRITE(numout,*) 'Mineralization exceeds litter pools in stomate_litter'
              WRITE(numout,*) 'This will cause problems later on'
              DO iele = 1,nelements
                 WRITE(numout,*) 'Structural litter pool, qd, ', &
                      ivm, ilev, iele, litter(:,istructural,ivm,ilev,iele),qd(:,iele)
              END DO
              CALL ipslerr(3,'stomate_litter','Structural mineralization exceeds litter pool',&
                   'This will cause problems later on','Fix the problem now')
              
           ENDIF

!!$        ! Store n_mineralization for later use   
!!$        n_mineralisation(:,ivm) = n_mineralisation(:,ivm) + qd(:,initrogen) 
           
           !+++++CHECK+++++ 
           ! After large-scale dieback events, so much soil mineral N becomes immobilized 
           ! to decompose litter that too little N is left for plant regrowth. To address 
           ! this, we implicitly represent the action of fungivores, which release N for 
           ! the plants and increase N turnover rates.  We set aside a fraction of qd, 
           ! n_fungivores, which becomes available for plant uptake in nitrogen_dynamics.      
           ! This parameter is not very well constrained and has been calibrated to find 
           ! a good compromise between plant regrowth and litter decomposition.

           n_mineralisation(:,ivm) = n_mineralisation(:,ivm) + (1-f_fungivores(:,ivm))*qd(:,initrogen)

           n_fungivores(:,ivm) = n_fungivores(:,ivm) + f_fungivores(:,ivm)*qd(:,initrogen)
          
           !! 5.1.3 non-lignin fraction of structural litter goes into
           !!       active (or surface) carbon pool + respiration 
           som_input(:,itarget,ivm,icarbon) = &
                som_input(:,itarget,ivm,icarbon) + & 
                frac_soil(istructural,itarget,ilev) * qd(:,icarbon) * &
                ( 1. - lignin_struc(:,ivm,ilev) ) / dt 
           
           ! Here resp_hetero_litter is divided by dt to 
           ! have a value which corresponds to the sechiba time step but 
           ! then in stomate.f90 resp_hetero_litter is multiplied by dt. 
           ! Perhaps it could be simplified.
           resp_hetero_litter(:,ivm) = resp_hetero_litter(:,ivm) + &
                ( 1. - frac_soil(istructural,itarget,ilev) ) * qd(:,icarbon) * &
                ( 1. - lignin_struc(:,ivm,ilev) ) / dt
      
           !! 5.1.4 lignin fraction of structural litter goes into
           !!       slow carbon pool + respiration
           som_input(:,islow,ivm,icarbon) = som_input(:,islow,ivm,icarbon) + &
                frac_soil(istructural,islow,ilev) * qd(:,icarbon) * &
                lignin_struc(:,ivm,ilev) / dt 
           
           ! BE CAREFUL: Here resp_hetero_litter is divided by dt to have a
           ! value which corresponds to the sechiba time step but then in 
           ! stomate.f90 resp_hetero_litter is multiplied by dt. Perhaps it 
           ! could be simplified. Moreover, we must totally adapt the 
           ! routines to the dt_sechiba/one_day time step and avoid some 
           ! constructions that could create bug during future developments.
           resp_hetero_litter(:,ivm) = resp_hetero_litter(:,ivm) + &
                ( 1. - frac_soil(istructural,islow,ilev) ) * qd(:,icarbon) * &
                lignin_struc(:,ivm,ilev) / dt
           
           ! Debug
           IF(printlev_loc.GE.4 .AND. ivm.EQ.test_pft)THEN
              WRITE(numout,*) 'check 1, ',ilev
              WRITE(numout,*) 'dead_leaves(istructural), ',&
                   dead_leaves(:,test_pft,istructural)
              WRITE(numout,*) 'qd, icarbon, ilev', ilev, qd(:,icarbon)
              WRITE(numout,*) 'resp_hetero, ',resp_hetero_litter(:,test_pft)
              WRITE(numout,*) 'som_input - C, ', &
                   SUM(som_input(:,:,test_pft,icarbon),2)
              WRITE(numout,*) 'n_mineralisation, ',n_mineralisation(:,test_pft)
              WRITE(numout,*) 'som_input - N, ',&
                   SUM(som_input(:,:,test_pft,initrogen),2)
              WRITE(numout,*) ''
           ENDIF
           !-
        
           !! 5.2 metabolic litter goes into active carbon pool + respiration
           !! 5.2.1 total quantity of metabolic litter that is decomposed
           fd(:) = dt*litter_dec_fac(imetabolic) * control_temp(:,ilev) * &
                control_moist(:,ilev)
           
           ! Ensure that fd is between 0 and 1
           fd(:) = MAX(zero,MIN(fd(:),un))

           ! Calculate qd, make sure it has a positive value
           err(:) = zero
           WHERE ((litter(:,imetabolic,ivm,ilev,icarbon).GE.zero) .AND. &
                (litter(:,imetabolic,ivm,ilev,initrogen).GE.zero))

              ! Calculate the quantity of metabolic litter that should
              ! be decomposed
              qd(:,icarbon) = litter(:,imetabolic,ivm,ilev,icarbon) * fd(:)
              qd(:,initrogen) = litter(:,imetabolic,ivm,ilev,initrogen) * fd(:)

           ELSEWHERE

              ! Keep track of how many errors there are
              err(:) = 1.

           ENDWHERE

           ! Check for errors 
           IF (SUM(err(:)) .GT. zero) THEN
              
              WRITE(numout,*) 'Number of errors found in icarbon: ,', SUM(err(:))
              WRITE(numout,*) 'Negative litter pools in stomate_litter'
              WRITE(numout,*) 'That does not make any sense'
              DO iele = 1,nelements
                 WRITE(numout,*) 'Metabolic litter pool, ', &
                      ivm, ilev, iele, litter(:,imetabolic,ivm,ilev,iele)
              END DO
              CALL ipslerr(3,'stomate_litter','Negative metabolic litter pools',&
                   'This should not be the case','')
              !-
              
           END IF

           ! Subtract the mineralization from the litter pool but make
           ! sure that no negative values for the litter pool are created
           err(:) = zero 
           WHERE ( (litter(:,imetabolic,ivm,ilev,icarbon).GE.qd(:,icarbon)) .AND. &
                (litter(:,imetabolic,ivm,ilev,initrogen).GE.qd(:,initrogen)))

              ! qd is calculated as litter(:,imetabolic,ivm,ilev,iele)*fd(:)
              ! as long as fd is positive (which was enforced above) the
              ! subtraction below should never result in a number that is
              ! less than zero (except for precision issues). fd is the same
              ! for carbon and nitrogen and therefore the ratio is preserved
              ! during mineralization. The test conditions should be
              ! satisfied for C and N at the same time.
              litter(:,imetabolic,ivm,ilev,icarbon) = &
                   litter(:,imetabolic,ivm,ilev,icarbon) - qd(:,icarbon)
              litter(:,imetabolic,ivm,ilev,initrogen) = &
                   litter(:,imetabolic,ivm,ilev,initrogen) - qd(:,initrogen)

           ELSEWHERE

              err(:) = 1.

           ENDWHERE

           ! Check for errors 
           IF (SUM(err(:)) .GT. zero) THEN
              
              WRITE(numout,*) 'Number of errors found: ,', SUM(err(:))
              WRITE(numout,*) 'Mineralization exceeds litter pools in stomate_litter'
              WRITE(numout,*) 'This will cause problems latter on'
              DO iele = 1,nelements
                 WRITE(numout,*) 'Metabolic litter pool, qd, ', &
                      ivm, ilev, iele, litter(:,imetabolic,ivm,ilev,iele),qd(:,iele)
              END DO
              CALL ipslerr(3,'stomate_litter','Metabolic mineralization exceeds litter pool',&
                   'This will cause problems later on','Fix the problem now')
              !-
              
           ENDIF

!!$        ! Store n_mineralization for later use
!!$        n_mineralisation(:,ivm) = n_mineralisation(:,ivm) + qd(:,initrogen) 

           !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!+++++CHECK+++++!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! 
           ! After large-scale dieback events, so much soil mineral N becomes immobilized 
           ! to decompose litter that too little N is left for plant regrowth. To address 
           ! this, we implicitly represent the action of fungivores, which release N for 
           ! the plants and increase N turnover rates.  We set aside a fraction of qd, 
           ! n_fungivores, which becomes available for plant uptake in nitrogen_dynamics.      
           ! This parameter is not very well constrained and has been calibrated to find 
           ! a good compromise between plant regrowth and litter decomposition.
           n_mineralisation(:,ivm) = n_mineralisation(:,ivm) + &
                (1-f_fungivores(:,ivm))*qd(:,initrogen)    
           n_fungivores(:,ivm) = n_fungivores(:,ivm) + &
                f_fungivores(:,ivm)*qd(:,initrogen)

           !! 5.2.2 decompose same fraction of metabolic part of dead leaves.
           !  to avoid a multiple (for ibelow and iabove) modification of 
           !  dead_leaves, we do this test to do this calcul only ones in 
           !  1,nlev loop
           IF (ilev == iabove) THEN
              
              dead_leaves(:,ivm,imetabolic) = &
                   dead_leaves(:,ivm,imetabolic) * ( 1. - fd(:) )

           ENDIF

           !! 5.2.3 put decomposed litter into active 
           !  (or surface) pool + respiration 
           som_input(:,itarget,ivm,icarbon) = &
                som_input(:,itarget,ivm,icarbon) + & 
                frac_soil(imetabolic,itarget,ilev) * qd(:,icarbon) / dt 
           
           ! BE CAREFUL: Here resp_hetero_litter is divided by dt to have a 
           ! value which corresponds to the sechiba time step but then in 
           ! stomate.f90 resp_hetero_litter is multiplied by dt. 
           ! Perhaps it could be simplified. Moreover, we must totally adapt 
           ! the routines to the dtradia/one_day time step and avoid some 
           ! constructions that could create bugs during future developments.
           resp_hetero_litter(:,ivm) = resp_hetero_litter(:,ivm) + &
                ( 1. - frac_soil(imetabolic,itarget,ilev) ) * qd(:,icarbon) / dt

           ! Debug
           IF(printlev_loc.GE.4 .AND. ivm.EQ.test_pft)THEN
              WRITE(numout,*) 'check 2, ',ilev
              WRITE(numout,*) 'qd, icarbon, ilev', ilev, qd(:,icarbon)
              WRITE(numout,*) 'resp_hetero, ',resp_hetero_litter(:,test_pft)
              WRITE(numout,*) 'som_input - C, ', &
                   SUM(som_input(:,:,test_pft,icarbon),2)
              WRITE(numout,*) 'n_mineralisation, ',n_mineralisation(:,test_pft)
              WRITE(numout,*) 'som_input - N, ',&
                   SUM(som_input(:,:,test_pft,initrogen),2)
              WRITE(numout,*) ''
           ENDIF
           !-

           !! 5.3 woody litter: goes into active and slow carbon 
           !  pools + respiration
           
           !! 5.3.1 total quantity of woody litter which is decomposed
           fd(:) = dt*litter_dec_fac(iwoody) * &
                control_temp(:,ilev) * control_moist(:,ilev) * &
                EXP( -3. * lignin_wood(:,ivm,ilev) )
           
           ! Ensure that fd is between 0 and 1
           fd(:) = MAX(zero,MIN(fd(:),un))

           ! Calculate qd for woody litter
           err(:) = zero
           WHERE ((litter(:,iwoody,ivm,ilev,icarbon).GE.zero) .AND. &
                (litter(:,iwoody,ivm,ilev,initrogen).GE.zero))

                 ! Calculate the quantity of woody litter that should
                 ! be decomposed
                 qd(:,icarbon) = litter(:,iwoody,ivm,ilev,icarbon) * fd(:)
                 qd(:,initrogen) = litter(:,iwoody,ivm,ilev,initrogen) * fd(:)

           ELSEWHERE

              ! Keep track of how many errors there are
              err(:) = 1.

           ENDWHERE

           ! Check for errors 
           IF (SUM(err(:)) .GT. zero) THEN
              
              WRITE(numout,*) 'Number of errors found: ,', SUM(err(:))
              WRITE(numout,*) 'Negative litter pools in stomate_litter'
              WRITE(numout,*) 'That does not make any sense'
              DO iele = 1,nelements
                 WRITE(numout,*) 'Woody litter pool, ', &
                      ivm, ilev, iele, litter(:,iwoody,ivm,ilev,iele)
              END DO
              CALL ipslerr(3,'stomate_litter','Negative woody litter pools',&
                   'This should not be the case','')
              !-
              
           END IF

           ! Subtract the mineralization from the litter pool but make
           ! sure that no negative values for the litter pool are created
           err(:) = zero 
           WHERE ( (litter(:,iwoody,ivm,ilev,icarbon).GE.qd(:,icarbon)) .AND. &
                (litter(:,iwoody,ivm,ilev,initrogen).GE.qd(:,initrogen)))

              ! qd is calculated as litter(:,iwoody,ivm,ilev,iele)*fd(:)
              ! as long as fd is positive (which was enforced above) the
              ! subtraction below should never result in a number that is
              ! less than zero (except for precision issues). fd is the same
              ! for carbon and nitrogen and therefore the ratio is preserved
              ! during mineralization. The test conditions should be
              ! satisfied for C and N at the same time.
              litter(:,iwoody,ivm,ilev,icarbon) = &
                      litter(:,iwoody,ivm,ilev,icarbon) - qd(:,icarbon)
              litter(:,iwoody,ivm,ilev,initrogen) = &
                      litter(:,iwoody,ivm,ilev,initrogen) - qd(:,initrogen)

           ELSEWHERE

              err(:) = 1.

           ENDWHERE

           ! Check for errors 
           IF (SUM(err(:)) .GT. zero) THEN
              
              WRITE(numout,*) 'Number of errors found: ,', SUM(err(:))
              WRITE(numout,*) 'Mineralization exceeds litter pools in stomate_litter'
              WRITE(numout,*) 'This will cause problems latter on'
              DO iele = 1,nelements
                 WRITE(numout,*) 'Woody litter pool, qd, ', &
                      ivm, ilev, iele, litter(:,iwoody,ivm,ilev,iele),qd(:,iele)
              END DO
              CALL ipslerr(3,'stomate_litter','Woody mineralization exceeds litter pool',&
                   'This will cause problems later on','Fix the problem now')
              !-
              
           ENDIF

!!$        ! Store n_mineralization for later use
!!$        n_mineralisation(:,ivm) = n_mineralisation(:,ivm) + qd(:,initrogen)

           !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!+++++CHECK+++++!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! 
           ! After large-scale dieback events, so much soil mineral N becomes immobilized 
           ! to decompose litter that too little N is left for plant regrowth. To address 
           ! this, we implicitly represent the action of fungivores, which release N for 
           ! the plants and increase N turnover rates.  We set aside a fraction of qd, 
           ! n_fungivores, which becomes available for plant uptake in nitrogen_dynamics.      
           ! This parameter is not very well constrained and has been calibrated to find 
           ! a good compromise between plant regrowth and litter decomposition.

           n_mineralisation(:,ivm) = n_mineralisation(:,ivm) + (1-f_fungivores(:,ivm))*qd(:,initrogen)
         
           n_fungivores(:,ivm) = n_fungivores(:,ivm) +f_fungivores(:,ivm)*qd(:,initrogen)

           !! 5.3.2 non-lignin fraction of woody litter goes into
           !!       active/structural carbon pool + respiration (per time unit)
           som_input(:,itarget,ivm,icarbon) = som_input(:,itarget,ivm,icarbon) + &
                frac_soil(iwoody,itarget,ilev) * qd(:,icarbon) * &
                ( 1. - lignin_wood(:,ivm,ilev) ) / dt 
    
           resp_hetero_litter(:,ivm) = resp_hetero_litter(:,ivm) + &
                ( 1. - frac_soil(iwoody,itarget,ilev) ) * qd(:,icarbon) * &
                ( 1. - lignin_wood(:,ivm,ilev) ) / dt
    
           !! 5.3.3 lignin fraction of woody litter goes into
           !!       slow carbon pool + respiration (per time unit)
           som_input(:,islow,ivm,icarbon) = som_input(:,islow,ivm,icarbon) + &
                frac_soil(iwoody,islow,ilev) * qd(:,icarbon) * &
                lignin_wood(:,ivm,ilev) / dt 

           resp_hetero_litter(:,ivm) = resp_hetero_litter(:,ivm) + &
                ( 1. - frac_soil(iwoody,islow,ilev) ) * qd(:,icarbon) * &
                lignin_wood(:,ivm,ilev) / dt

           ! Debug
           IF(printlev_loc.GE.4 .AND. ivm.EQ.test_pft)THEN
              WRITE(numout,*) 'check 3, ',ilev
              WRITE(numout,*) 'qd, icarbon, ilev', ilev, qd(:,icarbon)
              WRITE(numout,*) 'resp_hetero, ',resp_hetero_litter(:,test_pft)
              WRITE(numout,*) 'som_input - C, ', &
                   SUM(som_input(:,:,test_pft,icarbon),2)
              WRITE(numout,*) 'n_mineralisation, ',n_mineralisation(:,test_pft)
              WRITE(numout,*) 'som_input - N, ',&
                   SUM(som_input(:,:,test_pft,initrogen),2)
              WRITE(numout,*) ''
           ENDIF
           !-

        ENDDO

     ENDDO

     ! 5.4 Calculate som_input for nitrogen
     ! This far the som input for icarbon has been recorded. Now
     ! use the CN ratios to calculate the som input for nitrogen.
     ! Note the difference in units between ::som_input (gN m-2 s-1) 
     ! and ::n_mineralisation (gN m-2).
     som_input(:,iactive,:,initrogen) = &
         som_input(:,iactive,:,icarbon)/CN_target(:,:,iactive)  
     som_input(:,islow,:,initrogen) = &
         som_input(:,islow,:,icarbon)/CN_target(:,:,islow) 
     som_input(:,isurface,:,initrogen) = &
         som_input(:,isurface,:,icarbon)/CN_target(:,:,isurface) 

     ! We may need these later:
     DO ipts = 1, npts
        
        DO ivm = 1, nvm

           n_min_old(ipts,ivm) = n_mineralisation(ipts,ivm)
           som_input_total(ivm,initrogen) = &
                som_input(ipts,iactive,ivm,initrogen) + &
                som_input(ipts,islow,ivm,initrogen) + &
                som_input(ipts,isurface,ivm,initrogen)

           IF (som_input_total(ivm,initrogen).GT.zero) THEN

              share_som_input_act_n(ipts,ivm) = &
                   som_input(ipts,iactive,ivm,initrogen) &
                   / som_input_total(ivm,initrogen)
              share_som_input_slow_n(ipts,ivm) = &
                   som_input(ipts,islow,ivm,initrogen) &
                   / som_input_total(ivm,initrogen)
              share_som_input_surf_n(ipts,ivm) = &
                   som_input(ipts,isurface,ivm,initrogen) &
                   / som_input_total(ivm,initrogen)
           ENDIF
           
        ENDDO

     ENDDO

     ! 5.5 Truncate som_input(:,:,:,initrogen) if necessary. 

     ! 5.5.1 We first need to pre-determine the value of n_mineralisation(:,:)
     ! we will have in the next routine (som_dynamics) after the fluxes are 
     ! subtracted. This means we have to port some of the code from som_dynamics 
     ! to here. 
     ! The following block of code is copied from sections 1.1 - 3.2.3 of 
     ! som_dynamics:

     !! Initialise
     resp_hetero_soil(:,:) = zero
     total_som_n_flux(:,:,:) = zero
     n_mineralisation_som(:,:) = zero
     frac_carb(:,:,:) = zero
     som_temp(:,:,:,:) = zero
     n_min_new(:,:)=zero

     ! Get soil "constants"
     ! Flux fractions between carbon pools: depend on clay content,
     ! recalculated each time
     ! From active pool: depends on clay content
     frac_carb(:,iactive,ipassive) = active_to_pass_ref_frac + &
          active_to_pass_clay_frac*clay(:)
     frac_carb(:,iactive,islow) = un - frac_carb(:,iactive,ipassive) - &
          (active_to_co2_ref_frac - active_to_co2_clay_silt_frac*(clay(:)+silt(:)))

     ! From slow pool    
     frac_carb(:,islow,ipassive) = slow_to_pass_ref_frac + &
          slow_to_pass_clay_frac*clay(:)

     ! OCN doesn't use Parton 1993 formulation for
     ! frac_carb(:,islow,ipassive) 
     ! but the one of 1987 : ie = 0.03 ....
     frac_carb(:,islow,iactive) = un - frac_carb(:,islow,ipassive) - &
          slow_to_co2_ref_frac

     ! From passive pool
     frac_carb(:,ipassive,iactive) = pass_to_active_ref_frac
     frac_carb(:,ipassive,islow) = pass_to_slow_ref_frac

     ! From surface pool
     frac_carb(:,isurface,islow) = surf_to_slow_ref_frac

     !! Determine the respiration fraction : what's left after
     ! subtracting all the 'pool-to-pool' flux fractions
     ! Diagonal elements of frac_carb are zero
     frac_resp(:,:) = un - frac_carb(:,:,isurface) - frac_carb(:,:,iactive) - &
        frac_carb(:,:,islow) - frac_carb(:,:,ipassive)

     !! Turnover in SOM pools (in days)
     ! som_turn_ipool is the turnover (in years)
     ! It is weighted by Temp and Humidity function later
     som_turn(iactive) = som_turn_iactive / one_year
     som_turn(islow) = som_turn_islow / one_year
     som_turn(ipassive) = som_turn_ipassive / one_year
     som_turn(isurface) = som_turn_isurface / one_year

     !! Update the SOM stocks with the different soil carbon input
     som_temp(:,:,:,:) = som(:,:,:,:) + som_input(:,:,:,:) * dt

     !! Fluxes between carbon reservoirs, and to the atmosphere
     ! (respiration)
      
     !! Calculate fluxes
     !! Flux out of pools
     ! Loop over carbon pools from which the flux comes
     DO ivm = 1,nvm

        DO icarb = 1, ncarb
 
           DO  iele = 1, nelements
     
              ! Determine total flux out of pool [gN m-2]
              fluxtot(:,ivm,icarb,iele) = dt*som_turn(icarb) * &
                   som_temp(:,icarb,ivm,iele) * control_moist(:,ibelow) * &
                   control_temp(:,ibelow) * decomp_factor(ivm)
              
              ! Flux from active pools depends on clay content [gN m-2]
              IF ( icarb .EQ. iactive ) THEN
                 fluxtot(:,ivm,icarb,iele) = fluxtot(:,ivm,icarb,iele) * &
                      ( un - som_turn_iactive_clay_frac * clay(:) )
              ENDIF
 
              ! Update the loss in each carbon pool [gN m-2]
              som_temp(:,icarb,ivm,iele) = som_temp(:,icarb,ivm,iele) - &
                   fluxtot(:,ivm,icarb,iele)
     
           ENDDO ! nelements
     
           ! Fluxes towards the other pools (icarb -> iicarb)
           ! Loop over the SOM pools where the flux goes
           DO iicarb = 1, ncarb
     
              ! Carbon flux [gN m-2]
              flux(:,ivm,icarb,iicarb,icarbon) = frac_carb(:,icarb,iicarb) * &
                   fluxtot(:,ivm,icarb,icarbon)
     
              ! Nitrogen flux - Function of the C stock of the 'departure'
              ! pool and of the C to N target ratio of the 'arrival' pool [gN
              ! m-2]
              flux(:,ivm,icarb,iicarb,initrogen) = frac_carb(:,icarb,iicarb) * &
                   fluxtot(:,ivm,icarb,icarbon) / &
                   CN_target(:,ivm,iicarb)
     
           ENDDO ! receiving SOM pools
     
        ENDDO ! donating SOM pools

     ENDDO ! ivm

     !! Respiration
     ! Fluxtot is in [gN m-2], resp_hetero_soil is in [gN m-2 dt-1] 
     ! divide by dt.
     !+++CHECK+++
     ! The ENDDO above and the DO ivm below could probably be removed
     ! to make a single bigger loop.
     !+++++++++++
     DO ivm = 1,nvm

        resp_hetero_soil(:,ivm) = &
             ( frac_resp(:,iactive) * fluxtot(:,ivm,iactive,icarbon) + &
             frac_resp(:,islow) * fluxtot(:,ivm,islow,icarbon) + &
             frac_resp(:,ipassive) * fluxtot(:,ivm,ipassive,icarbon) + &
             frac_resp(:,isurface) * fluxtot(:,ivm,isurface,icarbon) ) / dt

        !! Add fluxes to active, slow, and passive pools
        ! Loop over SOM pools

        DO icarb = 1, ncarb

           ! Calculate the components of the som fluxes [gN m-2]
           ! or [gC m-2]
           som_temp(:,icarb,ivm,initrogen) = som_temp(:,icarb,ivm,initrogen) + &
                flux(:,ivm,iactive,icarb,initrogen) + &
                flux(:,ivm,ipassive,icarb,initrogen) + &
                flux(:,ivm,islow,icarb,initrogen) + &
                flux(:,ivm,isurface,icarb,initrogen)

           ! Calculate the total flux [gN m-2]
           total_som_n_flux(:,ivm,initrogen) = total_som_n_flux(:,ivm,initrogen) + &
                flux(:,ivm,iactive,icarb,initrogen) + &
                flux(:,ivm,ipassive,icarb,initrogen) + &
                flux(:,ivm,islow,icarb,initrogen) + &
                flux(:,ivm,isurface,icarb,initrogen)

        ENDDO ! Loop over SOM pools

     ENDDO

!!$    ! 5.5.2 n_mineralisation_som(ipts,ivm) is what we're after. This value 
!!$    ! should not become negative, or else we may deplete our soil_n_min pools.
!!$    DO ivm = 1,nvm
!!$       DO ipts = 1,npts
!!$          n_mineralisation_som(ipts,ivm) = n_mineralisation(ipts,ivm) + &
!!$               SUM(fluxtot(ipts,ivm,:,initrogen)) - &
!!$               total_som_n_flux(ipts,ivm,initrogen)
!!$       ENDDO
!!$    ENDDO      

     ! 5.5.3 Calculate the final value for mineralisation
     DO ivm = 1, nvm

        DO ipts = 1, npts

           ! No redistrubtion needed unless specified in the code
           ld_redistribute(ivm) = zero

           ! Only check for existing PFTs
           IF (veget_cov_max(ipts,ivm).LE.zero) CYCLE
 
           ! Total internal nitrogen flux to be taken from the 
           ! mineralised pool. Note that litter decomposition does not
           ! contribute directly to the passive component of the soil 
           ! organic matter [gN m-2 dt-1] multiply with dt to obtain 
           ! [gN m-2].
           som_input_total(ivm,:) = (som_input(ipts,iactive,ivm,:) + & 
                som_input(ipts,islow,ivm,:) + & 
                som_input(ipts,isurface,ivm,:))*dt

           ! Store the positive component of n_mineralisation calculated
           ! above
           n_mineralisation_init = n_mineralisation(ipts,ivm)

           ! Calculate the n_mineralisation pool after all internal
           ! n fluxes from som_input(initrogen) are accounted for. 
           ! n_mineralisation can be both positive or negative
           n_mineralisation(ipts,ivm) = n_mineralisation_init - &
                som_input_total(ivm,initrogen)

           n_mineralisation_som(ipts,ivm) = n_mineralisation(ipts,ivm) + &
                SUM(fluxtot(ipts,ivm,:,initrogen)) - &
                total_som_n_flux(ipts,ivm,initrogen)

           ! Check whether the decomposition estimates above are realistic in 
           ! terms of the nitrogen available for immobilisation if 
           ! n_mineralisation is negative. 
           IF ((soil_n_min(ipts,ivm,initrate)+soil_n_min(ipts,ivm,iammonium)) + &
              n_mineralisation_init.LT.zero) THEN

              CALL ipslerr(3,'stomate_litter',&
                   'First time that this problem is encountered',&
                   'Debug the code first before continuing','Really!')

              ! All the C and N contained in som_input will need to be moved 
              ! back into the litter pool. Furthermore the n_mineralisation
              ! needs to be decreased as is the heterotrophic respiration
              DO iele = 1,nelements
                 litter_total_new(iele) = &
                      SUM(SUM(litter(ipts,:,ivm,:,iele),1)) + som_input_total(ivm,iele)
                 som_input_total(ivm,iele) = zero
              ENDDO

              ! Adjust resp_hetero_litter proportionaly.
              delta_hetero_litter = (resp_hetero_litter(ipts,ivm)- &
                   (resp_hetero_litter(ipts,ivm)/n_mineralisation_init) * &
                   (soil_n_min(ipts,ivm,initrate)+soil_n_min(ipts,ivm,iammonium)))*dt

              ! Update the litter pools
              litter_total_new(initrogen) = litter_total_new(initrogen) + &
                   n_mineralisation_init - (soil_n_min(ipts,ivm,initrate)+soil_n_min(ipts,ivm,iammonium))
              litter_total_new(icarbon) = litter_total_new(icarbon) + delta_hetero_litter

              ! Set flag to redistribute the new pools
              ld_redistribute(ivm) = 1.

           ELSEIF ((((soil_n_min(ipts,ivm,initrate)+soil_n_min(ipts,ivm,iammonium)) + &
              n_mineralisation(ipts,ivm).LT.zero) .AND. &
              ((soil_n_min(ipts,ivm,initrate)+soil_n_min(ipts,ivm,iammonium)) + &
              n_mineralisation_init.GT.zero)) .OR. &
              (n_mineralisation_som(ipts,ivm) .LT. -min_stomate)) THEN

              ! If the above conditions are satified there exists a solution.
              ! The new n_mineralisation should get a value between its initial
              ! and its current value. 

              ! Initialize
              litter_total_new(:) = zero
              
              ! CN_ratio of som_input total
              IF (som_input_total(ivm,icarbon).GT.zero)THEN
                 
                 CN_input_total = som_input_total(ivm,initrogen) / &
                      som_input_total(ivm,icarbon)
                 
              ELSE
                 
                 WRITE(numout,*) 'ERROR: surprise in CN calculation &
                      & no carbon available, ', ivm, som_input_total(ivm,icarbon), &
                      som_input_total(ivm,initrogen)
                 CALL ipslerr(3,'stomate_litter','no carbon',&
                      'problems to calculate C/N ratio for som_input_total','')

              ENDIF

!!$           ! The final n_mineralisation calculated as the sum of
!!$           ! n_mineralisation_init - som_input_new should be the 
!!$           ! maximum that can be afforded which is the current active
!!$           ! soil nitrogen pool [gN m-2]. See nitrogen_dynamics.
!!$
!!$           n_mineralisation(ipts,ivm) = - (soil_n_min(ipts,ivm,initrate) + &
!!$                soil_n_min(ipts,ivm,iammonium))

              !!!!!!!!!!!!!!!!!!!!!!!!!!!!!+++++CHECK+++++!!!!!!!!!!!!!!!!!!!!!!!!!!!!
              ! The above formulation had allowed negative n_mineralisation values to 
              ! be passed to stomate_soilcarbon. Previously, we had treated these as 
              ! immobilization. If we have persistent negative values, however, we 
              ! can reach a situation where we completely deplete our soil_n_min 
              ! pools to satisfy the demand for immobilization. This tends to happen
              ! when there is no plant recruitment, in which case a generation 
              ! of trees will all reach their maximum height and die at the same 
              ! time. This creates an enormous amount of litter, meaning our som_input 
              ! values become relatively large and when we subtract som_input from 
              ! n_mineralisation, the values become negative.
              ! To remedy this, we'll truncate som_input based on the current 
              ! n_mineralisation pool, but we'll limit the occurrence of negative
              ! values. Instead we'll set a minimum (min_n) for n_mineralisation 
              ! (e.g. .00001 g/m^2) so we can keep some N in the soil. In the future, 
              ! we may be able to use the old formulation for simulations using
              ! recruitment. As of rev 4574 we were not using recruitment in the model. 
              
              IF(n_min_old(ipts,ivm).GT.min_n) THEN  

                 n_min_new(ipts,ivm) = min_n
                 som_input_new(ivm,initrogen) = n_min_old(ipts,ivm) - &
                      n_min_new(ipts,ivm)

              ELSE

                 !!+++++++++++++++++++++++++CHECK+++++++++++++++++++++++++++++++++
                 ! If n_mineralisation is less than min_n, take a fraction of the
                 ! remaining n_mineralisation for som_input. This is arbitrarily 
                 ! set to 50% for now. Needs further testing/discussion.
                 
                 som_input_new(ivm,initrogen) = 0.5*n_min_old(ipts,ivm)

              ENDIF
 
!!$           ! The adjusted som_input carbon [gC m-2] should be
!!$           som_input_new(ivm,icarbon) = som_input_new(ivm,initrogen) / CN_input_total

              !!!!!!!!!!!!!!!!!!!!!!!!!!!!!+++++CHECK+++++!!!!!!!!!!!!!!!!!!!!!!!!!!!!
              ! When too little N is available to reach the target C:N ratio of
              ! the receiving pool, the equation above results in values that are too 
              ! low for som_input_total(:,icarbon), meaning carbon litter can not 
              ! decompose. som_input_total(:,icarbon) still needs to be fixed, 
              ! however, or else the C:N ratio in the surface soil pool will become 
              ! unrealistic.
              ! Rather than setting som_input_new(:,icarbon) proportionally as a factor 
              ! of som_input_new(:,initrogen) and CN_input_total, we need to have enough
              ! som_input(:,:,:,icarbon) to decompose the litter while at the same time 
              ! constraining [som_input(:,:,:,icarbon)/som_input(:,:,:,initrogen)] to a
              ! reasonable range. For the time being, we'll do this by capping the 
              ! ratio at a prescribed value (max_cn).
              ! The ecological justification behind this is that mycorrhizae take up 
              ! extra C using the photosynthesis product. Thus, litter can still 
              ! decompose even though C is not incorporated into the plant biomass. 
              ! The C:N ratio controlling this flux can therefore be quite flexible.   

              som_input_new(ivm,icarbon) = som_input_total(ivm,icarbon)

              ! Protect against dividing by zero.  som_input_new(ivm,initrogen) may be zero here in cases
              ! cases where n_mineralisation is zero at the beginning of the run, and then there
              ! is some new litter input which forces us to this point in the code.  This can
              ! happen if there is no litter for a long time, but to date we have only seen
              ! this in cases where the GPP is zero for a long time.  So I will flag it as a 
              ! warning.
              IF(som_input_new(ivm,initrogen) .NE. zero)THEN
                 IF((som_input_new(ivm,icarbon)/som_input_new(ivm,initrogen)) .GT. max_cn) THEN
                    som_input_new(ivm,icarbon) = som_input_new(ivm,initrogen) * max_cn
                 ENDIF
              ELSE
                 ! leave the new soil organic matter carbon input as it was, but 
                 ! write a warning. This warning is not very useful for crops
                 ! so it is only written for trees.
                 IF (is_tree(ivm)) THEN
                    WRITE(numout,*) 'WARNING: ivm,icarbon,initrogen,ipts ',&
                         ivm,icarbon,initrogen,ipts
                    WRITE(numout,*) 'WARNING: som_input_new(ivm,initrogen), &
                         & n_min_old(ipts,ivm)',som_input_new(ivm,initrogen),&
                      n_min_old(ipts,ivm)
                 WRITE(numout,*) 'WARNING: n_min_new(ipts,ivm),&
                      n_mineralisation(ipts,ivm),som_input_new(ivm,icarbon)',&
                      n_min_new(ipts,ivm), n_mineralisation(ipts,ivm),&
                      som_input_new(ivm,icarbon)
                 CALL ipslerr(2,'stomate_litter','Perhaps no n_mineralization &
                      & pools, but new litter','Previously only seen in cases &
                      & where GPP is zero for long term','')
                 ENDIF !is_tree
              ENDIF

              ! Calculate the n_mineralisation pool after all internal
              ! n fluxes from som_input(initrogen) are accounted for. 
              ! n_mineralisation can be both positive or negative
              n_mineralisation(ipts,ivm) = n_min_old(ipts,ivm) - &
                   som_input_new(ivm,initrogen)


              ! Error checking
              IF (som_input_new(ivm,initrogen).LT.zero) THEN

                 WRITE(numout,*) 'ERROR: should not be here ',&
                      (soil_n_min(ipts,ivm,initrate)+soil_n_min(ipts,ivm,iammonium)),&
                      ivm, n_mineralisation(ipts,ivm),&
                      som_input_total(ivm,initrogen)
                 CALL ipslerr(3,'stomate_litter',&
                      'Seems that the nitrogen can be satisfied',&
                      'Should not have ended up in this part of the code','')

              ENDIF

              ! The nitrogen that was not mineralized should be added back 
              ! into the litter pool. The carbon no longer included in 
              ! som_input should also be added back into the litter pool.
              litter_total_new(initrogen) = total_init_nitrogen(ipts,ivm) - &
                   n_mineralisation(ipts,ivm) - som_input_new(ivm,initrogen) - &
                   n_fungivores(ipts,ivm)
              litter_total_new(icarbon) = SUM(SUM(litter(ipts,:,ivm,:,icarbon),1)) + &
                   som_input_total(ivm,icarbon) - som_input_new(ivm,icarbon)

              ! Som_input was adjusted to ensure that after accounting for
              ! n_mineralisation there is enough soil nitrogen left to
              ! account for nitrogen immobilisation. When litter is 
              ! transformed into som_input, part of the carbon is lost as
              ! heterotrophic respiration. For each unit of som_input a
              ! certain number given by ::resp_ratio is respired
              IF ((som_input_total(ivm,icarbon) .GT. zero) .AND. &
                   (resp_hetero_litter(ipts,ivm).GT.zero)) THEN
                 
                 ! The change will be used to adjust the litter pool. The units should
                 ! be [gC m-2]
                 delta_hetero_litter = &
                      (som_input_total(ivm,icarbon) - som_input_new(ivm,icarbon)) * &
                      (resp_hetero_litter(ipts,ivm)/som_input_total(ivm,icarbon))*dt

                 ! Respiration is a flux, the units should be [gC m-2 dt-1]
                 resp_hetero_litter(ipts,ivm) = som_input_new(ivm,icarbon) * &
                      resp_hetero_litter(ipts,ivm) / som_input_total(ivm,icarbon)

              ELSE

                 WRITE(numout,*) 'ERROR: surprise when recalculating &
                      & resp_hetero_litter, ', som_input_new(ivm,icarbon), &
                      resp_hetero_litter(ipts,ivm), som_input_total(ivm,icarbon)
                 CALL ipslerr(3,'stomate_litter','problems with the sign',&
                      'when calculating delta_hetero_litter','')

              ENDIF

              ! The carbon that is not respired stays in the litter pool.
              litter_total_new(icarbon) = litter_total_new(icarbon) + &
                   delta_hetero_litter

              ! Set flag to redistribute the new pools
              ld_redistribute(ivm) = 1.


           ELSEIF ((soil_n_min(ipts,ivm,initrate)+soil_n_min(ipts,ivm,iammonium)) + &
                n_mineralisation(ipts,ivm).GE.zero) THEN

              ! Set flag to redistribute the new pools
              ld_redistribute(ivm) = zero


           ELSE

              WRITE(numout,*) 'ERROR: unexpected case'
              WRITE(numout,*) 'mineralisation wanted, ',n_mineralisation(ipts,ivm)
              WRITE(numout,*) 'mineralisation_old, ',n_mineralisation_init
              WRITE(numout,*) 'soil_n_min, ',(soil_n_min(ipts,ivm,initrate)+soil_n_min(ipts,ivm,iammonium))
              CALL ipslerr(3,'stomate_litter','Unexpected case',&
                   'Check the logic of IF-statement','')

           ENDIF
 
           IF (ld_redistribute(ivm) == 1.) THEN
     
              ! Share of each component to the total flux
              DO iele = 1,nelements
     
                 IF (som_input_total(ivm,iele).GT.zero) THEN
     
                    share_som_input_active = som_input(ipts,iactive,ivm,iele) / &
                         (som_input_total(ivm,iele) / dt)
                    share_som_input_slow = som_input(ipts,islow,ivm,iele) / &
                         (som_input_total(ivm,iele) / dt)
                    share_som_input_surface = 1 - share_som_input_active - &
                         share_som_input_slow
     
                 ELSE
     
                    WRITE(numout,*) 'ERROR: new problem. som_input = zero &
                         & cannot redistribute the overspended nitrogen'
                    CALL ipslerr(3,'stomate_litter','som_input = zero',&
                         'cannot redistribute the overspended N or C','')
     
                 ENDIF
     
                 ! Error checking
                 IF(err_act.GT.1)THEN
     
                    IF ( abs(share_som_input_surface + share_som_input_active + &
                         share_som_input_slow - 1) .GT. min_stomate) THEN
                       WRITE(numout,*) 'ERROR: sum of shares should be 0 ', &
                            ipts, ivm, share_som_input_surface + &
                            share_som_input_active + share_som_input_slow
                       CALL ipslerr (3,'stomate_litter','sum of shares',&
                            'share_som_input_xxx','')
     
                    ENDIF
      
                 ENDIF ! err_act.GT.1
     
                 ! Store old som_inputs in a temporary variable 
                 DO icarb = 1, ncarb
                    som_input_old(icarb,iele) = som_input(ipts,icarb,ivm,iele)*dt
                 ENDDO
     
                 ! Truncate som_input values
                 som_input(ipts,iactive,ivm,iele) = (share_som_input_active * &
                      som_input_new(ivm,iele)) 
                 som_input(ipts,islow,ivm,iele) = (share_som_input_slow * &
                      som_input_new(ivm,iele)) 
                 som_input(ipts,isurface,ivm,iele) = (share_som_input_surface * &
                      som_input_new(ivm,iele))
     
              ENDDO
     
              ! Convert to the initial units to avoid problems when closing the mass balance
              som_input(ipts,:,ivm,:) = som_input(ipts,:,ivm,:) / dt

              ! Distribute delta_som_input over the litter pools
              ! Calculate the share of each litter pool.
              DO iele = 1,nelements
                 share_litter_struc_a(ipts,ivm,iele) = litter(ipts,istructural,ivm,iabove,iele) / &
                      SUM(SUM(litter(ipts,:,ivm,:,iele),1))
                 share_litter_struc_b(ipts,ivm,iele) = litter(ipts,istructural,ivm,ibelow,iele) / &
                      SUM(SUM(litter(ipts,:,ivm,:,iele),1))
                 share_litter_met_a(ipts,ivm,iele) = litter(ipts,imetabolic,ivm,iabove,iele) / &
                      SUM(SUM(litter(ipts,:,ivm,:,iele),1))
                 share_litter_met_b(ipts,ivm,iele) = litter(ipts,imetabolic,ivm,ibelow,iele) / &
                      SUM(SUM(litter(ipts,:,ivm,:,iele),1))
                 share_litter_woody_a(ipts,ivm,iele) = litter(ipts,iwoody,ivm,iabove,iele) / &
                      SUM(SUM(litter(ipts,:,ivm,:,iele),1))
                 share_litter_woody_b(ipts,ivm,iele) = un - (share_litter_struc_a(ipts,ivm,iele) + &
                      share_litter_struc_b(ipts,ivm,iele) + share_litter_met_a(ipts,ivm,iele) + &
                      share_litter_met_b(ipts,ivm,iele) + share_litter_woody_a(ipts,ivm,iele))

                 ! Correct precision errors
                 IF((ABS(share_litter_woody_b(ipts,ivm,iele)) .LT. min_stomate) .AND. &
                    (ABS(share_litter_woody_b(ipts,ivm,iele)) .GT. zero)) THEN
                    share_litter_woody_b(ipts,ivm,iele) = zero
                 ENDIF 

                 ! Error checking
                 IF (ABS((share_litter_struc_a(ipts,ivm,iele) + share_litter_struc_b(ipts,ivm,iele) + &
                      share_litter_met_a(ipts,ivm,iele) + share_litter_met_b(ipts,ivm,iele) + &
                      share_litter_woody_a(ipts,ivm,iele) + litter(ipts,iwoody,ivm,ibelow,iele) / &
                      SUM(SUM(litter(ipts,:,ivm,:,iele),1)))-1).GT.min_stomate)THEN
    
                    WRITE(numout,*) 'ERROR: share_litter differs from 1 , ', &
                         share_litter_struc_a(ipts,ivm,iele) + &
                         share_litter_struc_b(ipts,ivm,iele) + share_litter_met_a(ipts,ivm,iele) + &
                         share_litter_met_b(ipts,ivm,iele) + share_litter_woody_a(ipts,ivm,iele) + &
                         litter(ipts,iwoody,ivm,ibelow,iele) / SUM(SUM(litter(ipts,:,ivm,:,iele),1)) 
                    CALL ipslerr(3,'stomate_litter','share_litter differs from 1','','')
    
                 ENDIF

                 litter(ipts,istructural,ivm,iabove,iele) = share_litter_struc_a(ipts,ivm,iele) * &
                      litter_total_new(iele)
                 litter(ipts,istructural,ivm,ibelow,iele) = share_litter_struc_b(ipts,ivm,iele) * &
                      litter_total_new(iele)
                 litter(ipts,imetabolic,ivm,iabove,iele) = share_litter_met_a(ipts,ivm,iele) * &
                      litter_total_new(iele) 
                 litter(ipts,imetabolic,ivm,ibelow,iele) = share_litter_met_b(ipts,ivm,iele) * &
                      litter_total_new(iele)
                 litter(ipts,iwoody,ivm,iabove,iele) = share_litter_woody_a(ipts,ivm,iele) * &
                      litter_total_new(iele)
                 litter(ipts,iwoody,ivm,ibelow,iele) = share_litter_woody_b(ipts,ivm,iele) * &
                      litter_total_new(iele)

              ENDDO ! nelements

           ENDIF !ld_redistribute
              
        ENDDO ! nvm
        
     ENDDO ! npts

    !! Reset variables
    resp_hetero_soil(:,:) = zero
    total_som_n_flux(:,:,:) = zero

    !! 6. Check mass balance closure

    !! 6.1 Calculate components of the mass balance
    !  The carbon in turnover and bm_to_litter is moved to different pools but
    !  ::turnover and ::bm_to_litter are never updated. They should either be
    !  set to zero or not be included in the calculation of ::pool_end 
 
    IF (err_act.GT.1) THEN

       pool_end(:,:,:) = zero
       pool_end(:,:,initrogen) = pool_end(:,:,initrogen) + &
            n_mineralisation(:,:) * veget_cov_max(:,:)

       DO ivm = 1,nvm
          pool_end(:,ivm,initrogen) = pool_end(:,ivm,initrogen) + &
                n_fungivores(:,ivm) * veget_cov_max(:,ivm) 
       ENDDO

       DO iele = 1,nelements
          
          ! The litter pool
          DO ilitt = 1,nlitt
             DO ilev = 1,nlevs
                pool_end(:,:,iele) = pool_end(:,:,iele) + &
                     (litter(:,ilitt,:,ilev,iele) * veget_cov_max(:,:))
             ENDDO
          ENDDO
          
       ENDDO

       !! 6.2 Calculate mass balance
       check_intern(:,:,:,:) = zero                    
       check_intern(:,:,iland2atm,icarbon) = -un * resp_hetero_litter(:,:) * &
            veget_cov_max(:,:) * dt

       DO iele = 1,nelements
          check_intern(:,:,ilat2out,iele) = -un * &
               SUM(som_input(:,:,:,iele),2) * veget_cov_max(:,:) * dt
          check_intern(:,:,ipoolchange,iele) = -un * (pool_end(:,:,iele) - &
               pool_start(:,:,iele))
       ENDDO

       closure_intern = zero
       DO imbc = 1,nmbcomp
          DO iele=1,nelements
             ! Debug
             IF (printlev_loc>=4) WRITE(numout,*) &
                  'check_intern, test_pft,imbc, iele, ', test_pft,imbc, &
                  iele, check_intern(:,test_pft,imbc,iele)!/&
                 ! veget_cov_max(:,test_pft)
             !-
             closure_intern(:,:,iele) = closure_intern(:,:,iele) + &
                  check_intern(:,:,imbc,iele)
          ENDDO

       ENDDO

    
       !! 6.3 Write the verdict
       CALL check_mass_balance("littercalc", closure_intern, npts, pool_end, &
            pool_start, veget_cov_max, 'pft')
       
    ENDIF ! err_act.GT.1

  !! 7. calculate fraction of total soil covered by dead leaves

    CALL deadleaf (npts, veget_cov_max, dead_leaves, deadleaf_cover)

  !! 8. (Quasi-)Analytical Spin-up : Start filling MatrixA

    IF (spinup_analytic) THEN

       MatrixA(:,:,:,:) = zero
       VectorB(:,:,:) = zero
       
       
       DO ivm = 1,nvm

          !- MatrixA : carbon fluxes leaving the litter
          
          MatrixA(:,ivm ,istructural_above,istructural_above)= - dt*litter_dec_fac(istructural) * &
               control_temp(:,iabove) * control_moist(:,iabove) * exp( -litter_struct_coef * lignin_struc(:,ivm ,iabove) )
          
          MatrixA(:,ivm ,istructural_below,istructural_below) = - dt*litter_dec_fac(istructural) * &
               control_temp(:,ibelow) * control_moist(:,ibelow) * exp( -litter_struct_coef * lignin_struc(:,ivm ,ibelow) )
          
          MatrixA(:,ivm ,imetabolic_above,imetabolic_above) = - dt*litter_dec_fac(imetabolic) * & 
               control_temp(:,iabove) * control_moist(:,iabove)
          
          MatrixA(:,ivm ,imetabolic_below,imetabolic_below) = - dt*litter_dec_fac(imetabolic) * & 
               control_temp(:,ibelow) * control_moist(:,ibelow)
          
          ! Flux leaving the woody above litter pool :
          MatrixA(:, ivm , iwoody_above, iwoody_above) = - dt * litter_dec_fac(iwoody) * control_temp(:,iabove) * &
               control_moist(:,iabove) * exp( -3. * lignin_wood(:,ivm ,iabove) )

          ! Flux leaving the woody below litter pool :
          MatrixA(:, ivm , iwoody_below, iwoody_below) = - dt * litter_dec_fac(iwoody) * control_temp(:,ibelow) * &
               control_moist(:,ibelow) * exp( -3. * lignin_wood(:,ivm ,ibelow))

          ! Flux received by the carbon surface from the woody above litter pool :
          MatrixA(:, ivm , isurface_pool, iwoody_above) = frac_soil(iwoody, isurface, iabove) * & 
               dt *litter_dec_fac(iwoody) * & 
               control_temp(:,iabove) * &
               control_moist(:,iabove) *  &
               exp( -3. * lignin_wood(:,ivm ,iabove) ) * ( 1. -  lignin_wood(:,ivm ,iabove) ) 

          ! Flux received by the carbon active from the woody below litter pool :
          MatrixA(:, ivm , iactive_pool, iwoody_below) = frac_soil(iwoody, iactive, ibelow) * & 
               dt *litter_dec_fac(iwoody) * &
               control_temp(:,ibelow) * &
               control_moist(:,ibelow) * &
               exp( -3. * lignin_wood(:,ivm ,ibelow) ) * ( 1. -  lignin_wood(:,ivm ,ibelow) ) 

          ! Flux received by the carbon slow from the woody above litter pool :
          MatrixA(:, ivm , islow_pool, iwoody_above) = frac_soil(iwoody, islow, iabove) * &
               dt *litter_dec_fac(iwoody) * &
               control_temp(:,iabove) * &
               control_moist(:,iabove) * &
               exp( -3. * lignin_wood(:,ivm ,iabove) ) * lignin_wood(:,ivm ,iabove)

          ! Flux received by the carbon slow from the woody below litter pool :
          MatrixA(:, ivm , islow_pool, iwoody_below) =  frac_soil(iwoody, islow, ibelow) * & 
               dt *litter_dec_fac(iwoody) * & 
               control_temp(:,ibelow) * &
               control_moist(:,ibelow) * &
               exp( -3. * lignin_wood(:,ivm ,ibelow) ) * lignin_wood(:,ivm ,ibelow)

                    
          !- MatrixA : carbon fluxes between the litter and the pools (the rest of the matrix is filled in stomate_soilcarbon.f90)
          MatrixA(:,ivm ,isurface_pool,istructural_above) = frac_soil(istructural,isurface,iabove) * &
               dt*litter_dec_fac(istructural) * &                    
               control_temp(:,iabove) * control_moist(:,iabove) * & 
               exp( -litter_struct_coef * lignin_struc(:,ivm ,iabove) ) * &
               ( 1. - lignin_struc(:,ivm ,iabove) ) 
                         

          MatrixA(:,ivm ,iactive_pool,istructural_below) = frac_soil(istructural,iactive,ibelow) * &
               dt*litter_dec_fac(istructural) * &                     
               control_temp(:,ibelow) * control_moist(:,ibelow) * & 
               exp( -litter_struct_coef * lignin_struc(:,ivm ,ibelow) ) * &
               ( 1. - lignin_struc(:,ivm ,ibelow) ) 
          
          MatrixA(:,ivm ,isurface_pool,imetabolic_above) =  frac_soil(imetabolic,isurface,iabove) * &
               dt*litter_dec_fac(imetabolic) * control_temp(:,iabove) * control_moist(:,iabove) 
           
          MatrixA(:,ivm ,iactive_pool,imetabolic_below) =  frac_soil(imetabolic,iactive,ibelow) * &
               dt*litter_dec_fac(imetabolic) * control_temp(:,ibelow) * control_moist(:,ibelow)          
                    
          MatrixA(:,ivm ,islow_pool,istructural_above) = frac_soil(istructural,islow,iabove) * &
               dt*litter_dec_fac(istructural) * &                   
               control_temp(:,iabove) * control_moist(:,iabove) * &
               exp( -litter_struct_coef * lignin_struc(:,ivm ,iabove) )* &
               lignin_struc(:,ivm ,iabove)
          
          
          MatrixA(:,ivm ,islow_pool,istructural_below) = frac_soil(istructural,islow,ibelow) * &
               dt*litter_dec_fac(istructural) * &   
               control_temp(:,ibelow) * control_moist(:,ibelow) *  &
                  exp( -litter_struct_coef * lignin_struc(:,ivm ,ibelow) )* &
                  lignin_struc(:,ivm ,ibelow) 
          
          
          !- VectorB : carbon input -
          
          VectorB(:,ivm ,istructural_above) = litter_inc(:,istructural,ivm ,iabove,icarbon)
          VectorB(:,ivm ,istructural_below) = litter_inc(:,istructural,ivm ,ibelow,icarbon)
          VectorB(:,ivm ,imetabolic_above) = litter_inc(:,imetabolic,ivm ,iabove,icarbon)
          VectorB(:,ivm ,imetabolic_below) = litter_inc(:,imetabolic,ivm ,ibelow,icarbon)
          VectorB(:,ivm ,iwoody_above) = litter_inc(:,iwoody,ivm ,iabove,icarbon)
          VectorB(:,ivm ,iwoody_below) = litter_inc(:,iwoody,ivm ,ibelow,icarbon)

          IF (printlev>=4) WRITE(numout,*) 'We filled MatrixA and VectorB' 
          
          WHERE(litter(:,istructural,ivm ,iabove,initrogen) .GT. min_stomate)
             CN_som_litter_longterm(:,ivm ,istructural_above) = &
                  ( CN_som_litter_longterm(:,ivm ,istructural_above) * (tau_CN_longterm-dt) &
                  + litter(:,istructural,ivm ,iabove,icarbon)/litter(:,istructural,ivm ,iabove,initrogen) * dt)/ (tau_CN_longterm)
          ENDWHERE
          
          WHERE(litter(:,istructural,ivm ,ibelow,initrogen) .GT. min_stomate)
             CN_som_litter_longterm(:,ivm ,istructural_below) = &
                  ( CN_som_litter_longterm(:,ivm ,istructural_below) * (tau_CN_longterm-dt) &
                  + litter(:,istructural,ivm ,ibelow,icarbon)/litter(:,istructural,ivm ,ibelow,initrogen) * dt)/ (tau_CN_longterm)
          ENDWHERE

          WHERE(litter(:,imetabolic,ivm ,iabove,initrogen) .GT. min_stomate)
             CN_som_litter_longterm(:,ivm ,imetabolic_above) = &
                  ( CN_som_litter_longterm(:,ivm ,imetabolic_above) * (tau_CN_longterm-dt) &
                  + litter(:,imetabolic,ivm ,iabove,icarbon)/litter(:,imetabolic,ivm ,iabove,initrogen) * dt)/ (tau_CN_longterm)
          ENDWHERE

          WHERE(litter(:,imetabolic,ivm ,ibelow,initrogen) .GT. min_stomate)
             CN_som_litter_longterm(:,ivm ,imetabolic_below) = &
                  ( CN_som_litter_longterm(:,ivm ,imetabolic_below) * (tau_CN_longterm-dt) &
                  + litter(:,imetabolic,ivm ,ibelow,icarbon)/litter(:,imetabolic,ivm ,ibelow,initrogen) * dt)/ (tau_CN_longterm)
          ENDWHERE

          WHERE(litter(:,iwoody,ivm ,iabove,initrogen) .GT. min_stomate)
             CN_som_litter_longterm(:,ivm ,iwoody_above) = &
                  ( CN_som_litter_longterm(:,ivm ,iwoody_above) * (tau_CN_longterm-dt) &
                  + litter(:,iwoody,ivm ,iabove,icarbon)/litter(:,iwoody,ivm ,iabove,initrogen) * dt)/ (tau_CN_longterm)
          ENDWHERE
          
          WHERE(litter(:,iwoody,ivm ,ibelow,initrogen) .GT. min_stomate)
             CN_som_litter_longterm(:,ivm ,iwoody_below) = &
                  ( CN_som_litter_longterm(:,ivm ,iwoody_below) * (tau_CN_longterm-dt) &
                  + litter(:,iwoody,ivm ,ibelow,icarbon)/litter(:,iwoody,ivm ,ibelow,initrogen) * dt)/ (tau_CN_longterm)
          ENDWHERE

       ENDDO ! Loop over # PFTs

          
    ENDIF ! spinup analytic 

    IF (printlev>=4) WRITE(numout,*) 'Leaving littercalc'

  END SUBROUTINE littercalc


!! ==============================================================================================================================\n
!! SUBROUTINE   : deadleaf
!!
!>\BRIEF        This routine calculates the deadleafcover. 
!!
!! DESCRIPTION  : It first calculates the lai corresponding to the dead leaves (LAI) using 
!! the dead leaves carbon content (DL) the specific leaf area (sla) and the 
!! maximal coverage fraction of a PFT (veget_cov_max) using the following equations:
!! \latexonly
!! \input{deadleaf1.tex}
!! \endlatexonly
!! \n
!! Then, the dead leaf cover (DLC) is calculated as following:\n
!! \latexonly
!! \input{deadleaf2.tex}
!! \endlatexonly
!! \n
!! 
!! RECENT CHANGE(S) : None
!!
!! MAIN OUTPUT VARIABLE: ::deadleaf_cover
!! 
!! REFERENCE(S) : None
!!
!! FLOWCHART : None
!! \n
!_ ================================================================================================================================

  SUBROUTINE deadleaf (npts, veget_cov_max, dead_leaves, deadleaf_cover)

  !! 0. Variable and parameter declaration
    
    !! 0.1 Input variables

    INTEGER(i_std), INTENT(in)                          :: npts           !! Domain size - number of grid pixels (unitless)
    REAL(r_std), DIMENSION(npts,nvm,nlitt), INTENT(in)  :: dead_leaves    !! Dead leaves per ground unit area, per PFT, 
                                                                          !! metabolic and structural  
                                                                          !! @tex $(gC m^{-2})$ @endtex
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in)          :: veget_cov_max      !! PFT "Maximal" coverage fraction of a PFT defined in 
                                                                          !! the input vegetation map 
                                                                          !! @tex $(m^2 m^{-2})$ @endtex 
    
    !! 0.2 Output variables
    
    REAL(r_std), DIMENSION(npts), INTENT(out)           :: deadleaf_cover !! Fraction of soil covered by dead leaves over all PFTs
                                                                          !! (0-1, unitless)

    !! 0.3 Modified variables

    !! 0.4 Local variables

    REAL(r_std), DIMENSION(npts)                        :: dead_lai       !! LAI of dead leaves @tex $(m^2 m^{-2})$ @endtex
    INTEGER(i_std)                                      :: ivm            !! Index (unitless)
!_ ================================================================================================================================
    
  !! 1. LAI of dead leaves
  
    IF (printlev>=3) WRITE(numout,*) 'Entering deadleaf'
    
    ! Debug
    IF (printlev_loc>=4) THEN
       WRITE(numout,*) 'dead_leaves, imetabolic ',dead_leaves(:,:,imetabolic)
       WRITE(numout,*) 'dead_leaves, istructural ',dead_leaves(:,:,istructural)
    END IF
    !-

    dead_lai(:) = zero

    DO ivm = 1,nvm !Loop over PFTs
       dead_lai(:) = dead_lai(:) + biomass_to_lai(( dead_leaves(:,ivm,imetabolic) + dead_leaves(:,ivm,istructural) ),npts,ivm) &
            * veget_cov_max(:,ivm)
    ENDDO

  !! 2. fraction of soil covered by dead leaves
    
    IF (printlev_loc>=4) WRITE(numout,*) 'dead_lai, ', dead_lai(:)

    
    deadleaf_cover(:) = un - exp( - 0.5 * dead_lai(:) )

    IF (printlev>=4) WRITE(numout,*) 'Leaving deadleaf'

  END SUBROUTINE deadleaf


!! ================================================================================================================================
!! FUNCTION     : control_moist_func
!!
!>\BRIEF        Calculate moisture control for litter and soil C decomposition
!!
!! DESCRIPTION  : Calculate moisture control factor applied
!! to litter decomposition and to soil carbon decomposition in
!! stomate_soilcarbon.f90 using the following equation: \n
!! \latexonly
!! \input{control_moist_func1.tex}
!! \endlatexonly
!! \n
!! with M the moisture control factor and soilmoisutre, the soil moisture 
!! calculated in sechiba.
!! Then, the function is ranged between Moistcont_min and 1:\n
!! \latexonly
!! \input{control_moist_func2.tex}
!! \endlatexonly
!! \n
!! RECENT CHANGE(S) : None
!!
!! RETURN VALUE : ::moistfunc_result
!! 
!! REFERENCE(S) : None
!!
!! FLOWCHART : None
!! \n
!_ ================================================================================================================================
  
  FUNCTION control_moist_func (npts, moist_in) RESULT (moistfunc_result)

  !! 0. Variable and parameter declaration
    
    !! 0.1 Input variables
          
    INTEGER(i_std), INTENT(in)               :: npts                !! Domain size - number of grid pixel (unitless)
    REAL(r_std), DIMENSION(npts), INTENT(in) :: moist_in            !! relative humidity (unitless)

    !! 0.2 Output variables
   
    REAL(r_std), DIMENSION(npts)             :: moistfunc_result    !! Moisture control factor (0.25-1, unitless)

    !! 0.3 Modified variables

    !! 0.4 Local variables

!_ ================================================================================================================================

    moistfunc_result(:) = -moist_coeff(1) * moist_in(:) * moist_in(:) + moist_coeff(2)* moist_in(:) - moist_coeff(3)
    moistfunc_result(:) = MAX( moistcont_min, MIN( un, moistfunc_result(:) ) )

  END FUNCTION control_moist_func


!! ================================================================================================================================
!! FUNCTION     : control_temp_func
!!
!>\BRIEF        Calculate temperature control for litter and soild C decomposition
!!
!! DESCRIPTION  : Calculate temperature control factor applied
!! to litter decomposition and to soil carbon decomposition in
!! stomate_soilcarbon.f90 using the following equation: \n
!! \latexonly
!! \input{control_temp_func1.tex}
!! \endlatexonly
!! \n
!! with T the temperature control factor, temp the temperature in Kelvin of 
!! the air (for aboveground litter) or of the soil (for belowground litter 
!! and soil)
!! Then, the function is limited in its maximal range to 1:\n
!! \latexonly
!! \input{control_temp_func2.tex}
!! \endlatexonly
!! \n
!! RECENT CHANGE(S) : None
!!
!! RETURN VALUE: ::tempfunc_result
!!
!! REFERENCE(S) : None
!!
!! FLOWCHART : None
!! \n
!_ ================================================================================================================================

  FUNCTION control_temp_func (npts, temp_in) RESULT (tempfunc_result)

  !! 0. Variable and parameter declaration
    
    !! 0.1 Input variables
    INTEGER(i_std), INTENT(in)                 :: npts            !! Domain size - number of land pixels (unitless)
    REAL(r_std), DIMENSION(npts), INTENT(in)   :: temp_in         !! Temperature (K)

    !! 0.2 Output variables
    REAL(r_std), DIMENSION(npts)               :: tempfunc_result !! Temperature control factor (0-1, unitless)

    !! 0.3 Modified variables

    !! 0.4 Local variables

!_ ================================================================================================================================

    tempfunc_result(:) = exp( soil_Q10 * ( temp_in(:) - (ZeroCelsius+tsoil_ref)) / Q10 )
    tempfunc_result(:) = MIN( un, tempfunc_result(:) )

  END FUNCTION control_temp_func

END MODULE stomate_litter
